@@ -2,7 +2,7 @@
 **      cdecl -- C gibberish translator
 **      src/c_ast_util.c
 **
-**      Copyright (C) 2017-2023  Paul J. Lucas
+**      Copyright (C) 2017-2024  Paul J. Lucas
 **
 **      This program is free software: you can redistribute it and/or modify
 **      it under the terms of the GNU General Public License as published by
@@ -30,16 +30,17 @@
 #define C_AST_UTIL_H_INLINE _GL_EXTERN_INLINE
 /// @endcond
 #include "c_ast_util.h"
+#include "c_ast.h"
+#include "c_kind.h"
 #include "c_operator.h"
-#include "c_typedef.h"
-#include "gibberish.h"
-#include "literals.h"
-#include "print.h"
+#include "c_type.h"
+#include "util.h"
 
 /// @cond DOXYGEN_IGNORE
 
 // standard
 #include <assert.h>
+#include <stddef.h>                     /* for NULL, size_t */
 
 /// @endcond
 
@@ -47,32 +48,27 @@
 NODISCARD
 static c_ast_t* c_ast_append_array( c_ast_t*, c_ast_t*, c_ast_t* );
 
+/**
+ * @addtogroup ast-functions-group
+ * @{
+ */
+
 ////////// local functions ////////////////////////////////////////////////////
 
 /**
- * Helper function for c_ast_add_array() that adds an array to the AST being
- * built.
- *
- * @param ast The AST to append to; may be NULL.
- * @param array_ast The array AST to append.  Its `of_ast` must be of kind
- * #K_PLACEHOLDER.
- * @param of_ast The AST to become the `of_ast` of \a array_ast.
- * @return Returns the AST to be used as the grammar production's return value.
- *
- * @sa c_ast_append_array()
+ * @copydoc c_ast_add_array
+ * @sa c_ast_add_array()
  */
 NODISCARD
 static c_ast_t* c_ast_add_array_impl( c_ast_t *ast, c_ast_t *array_ast,
                                       c_ast_t *of_ast ) {
+  assert( ast != NULL );
   assert( array_ast != NULL );
   assert( array_ast->kind == K_ARRAY );
   assert( array_ast->array.of_ast != NULL );
   assert( array_ast->array.of_ast->kind == K_PLACEHOLDER );
   assert( of_ast != NULL );
 
-  if ( ast == NULL )
-    return array_ast;
-
   switch ( ast->kind ) {
     case K_ARRAY:
       return c_ast_append_array( ast, array_ast, of_ast );
@@ -105,7 +101,7 @@ static c_ast_t* c_ast_add_array_impl( c_ast_t *ast, c_ast_t *array_ast,
         // else we'd end up with a "pointer to array 5 of array 3 of pointer to
         // int."
         //
-        PJL_IGNORE_RV(
+        PJL_DISCARD_RV(
           c_ast_add_array_impl( ast->ptr_ref.to_ast, array_ast, of_ast )
         );
         return ast;
@@ -154,8 +150,10 @@ static c_ast_t* c_ast_add_array_impl( c_ast_t *ast, c_ast_t *array_ast,
 /**
  * Helper function for c_ast_add_array_impl().
  * If \a ast is:
- *  + Not an array, makes \a array_ast an array of \a ast.
- *  + An array, appends \a array_ast to the end of the array AST chain.
+ *  + Not of \ref c_ast::kind "kind" #K_ARRAY, makes \a array_ast an array of
+ *    \a ast.
+ *  + Of \ref c_ast::kind "kind" #K_ARRAY, appends \a array_ast to the end of
+ *    the array AST chain.
  *
  * For example, given:
  *
@@ -167,10 +165,11 @@ static c_ast_t* c_ast_add_array_impl( c_ast_t *ast, c_ast_t *array_ast,
  *  + `array 3 of array 5 of array 7 of int`
  *
  * @param ast The AST to append to.
- * @param array_ast The array AST to append.  Its `of_ast` must be of kind
- * #K_PLACEHOLDER.
- * @param of_ast The AST to become the `of_ast` of \a array_ast.
- * @return If \a ast is an array, returns \a ast; otherwise returns \a
+ * @param array_ast The #K_ARRAY AST to append.  Its \a ref c_array_ast::of_ast
+ * "of_ast" _must_ be of kind #K_PLACEHOLDER.
+ * @param of_ast The AST to become the \a ref c_array_ast::of_ast "of_ast" of
+ * \a array_ast.
+ * @return If \a ast is of kind #K_ARRAY, returns \a ast; otherwise returns \a
  * array_ast.
  *
  * @sa c_ast_add_array_impl()
@@ -200,7 +199,8 @@ static c_ast_t* c_ast_append_array( c_ast_t *ast, c_ast_t *array_ast,
         break;
       FALLTHROUGH;
 
-    case K_ARRAY: {
+    case K_ARRAY:
+      NO_OP;
       //
       // On the next-to-last recursive call, this sets this array to be an
       // array of the new array; for all prior recursive calls, it's a no-op.
@@ -209,7 +209,6 @@ static c_ast_t* c_ast_append_array( c_ast_t *ast, c_ast_t *array_ast,
         c_ast_append_array( ast->array.of_ast, array_ast, of_ast );
       c_ast_set_parent( child_ast, ast );
       return ast;
-    }
 
     default:
       /* suppress warning */;
@@ -225,14 +224,8 @@ static c_ast_t* c_ast_append_array( c_ast_t *ast, c_ast_t *array_ast,
 }
 
 /**
- * Adds a function-like AST to the AST being built.
- *
- * @param ast The AST to append to.
- * @param func_ast The function-like AST to append.  Its `ret_ast` must be
- * NULL.
- * @param ret_ast The AST to become the `ret_ast` of \a func_ast.  Must be NULL
- * only when \a func_ast is a #K_CONSTRUCTOR.
- * @return Returns the AST to be used as the grammar production's return value.
+ * @copydoc c_ast_add_func()
+ * @sa c_ast_add_func()
  */
 NODISCARD
 static c_ast_t* c_ast_add_func_impl( c_ast_t *ast, c_ast_t *func_ast,
@@ -240,11 +233,11 @@ static c_ast_t* c_ast_add_func_impl( c_ast_t *ast, c_ast_t *func_ast,
   assert( ast != NULL );
   assert( func_ast != NULL );
   assert( is_1_bit_only_in_set( func_ast->kind, K_ANY_FUNCTION_LIKE ) );
+  assert( (func_ast->kind & K_ANY_FUNCTION_RETURN) == 0 || ret_ast != NULL );
+  assert( (func_ast->kind & K_ANY_FUNCTION_RETURN) != 0 || ret_ast == NULL );
   assert( func_ast->func.ret_ast == NULL );
-  assert( func_ast->kind != K_CONSTRUCTOR || ret_ast == NULL );
-  assert( func_ast->kind == K_CONSTRUCTOR || ret_ast != NULL );
 
-  if ( (ast->kind & (K_ARRAY | K_ANY_POINTER | K_ANY_REFERENCE)) != 0 ) {
+  if ( c_ast_is_parent( ast ) && ast->parent.of_ast != NULL ) {
     switch ( ast->parent.of_ast->kind ) {
       case K_ARRAY:
       case K_POINTER:
@@ -252,14 +245,19 @@ static c_ast_t* c_ast_add_func_impl( c_ast_t *ast, c_ast_t *func_ast,
       case K_REFERENCE:
       case K_RVALUE_REFERENCE:
         if ( ast->depth > func_ast->depth ) {
-          PJL_IGNORE_RV(
+          PJL_DISCARD_RV(
             c_ast_add_func_impl( ast->ptr_ref.to_ast, func_ast, ret_ast )
           );
           return ast;
         }
         FALLTHROUGH;
 
-      default:
+      case K_BUILTIN:
+      case K_CLASS_STRUCT_UNION:
+      case K_CONCEPT:
+      case K_ENUM:
+      case K_STRUCTURED_BINDING:
+      case K_TYPEDEF:
         if ( ast->kind == K_ARRAY ) {
           //
           // Before:
@@ -289,10 +287,34 @@ static c_ast_t* c_ast_add_func_impl( c_ast_t *ast, c_ast_t *func_ast,
       case K_APPLE_BLOCK:
         c_ast_set_parent( ret_ast, func_ast );
         return ast;
+
+      case K_FUNCTION:
+        //
+        // This is for a case like:
+        //
+        //      int f() ()
+        //
+        // that is, "function returning function" -- which is illegal (since
+        // functions can't return functions) and will be caught by
+        // c_ast_check_ret_type().  For now, do nothing.
+        break;
+
+      case K_CAPTURE:
+      case K_CAST:
+      case K_CONSTRUCTOR:
+      case K_DESTRUCTOR:
+      case K_LAMBDA:
+      case K_NAME:
+      case K_OPERATOR:
+      case K_UDEF_CONV:
+      case K_UDEF_LIT:
+      case K_VARIADIC:
+        UNEXPECTED_INT_VALUE( ast->parent.of_ast->kind );
     } // switch
   }
 
-  c_ast_set_parent( ret_ast, func_ast );
+  if ( c_ast_is_parent( func_ast ) )
+    c_ast_set_parent( ret_ast, func_ast );
   return func_ast;
 }
 
@@ -324,23 +346,24 @@ static c_ast_t const* c_ast_is_tid_any_qual_impl( c_ast_t const *ast,
 /**
  * Takes the storage (and attributes), if any, away from \a ast
  * (with the intent of giving them to another AST).
- * This is used is cases like:
- * ```
- * explain static int f()
- * ```
+ *
+ * @remarks This is used is cases like:
+ *
+ *      explain static int f()
+ *
  * that should be explained as:
- * ```
- * declare f as static function () returning integer
- * ```
+ *
+ *      declare f as static function () returning integer
+ *
  * and _not_:
- * ```
- * declare f as function () returning static integer
- * ```
+ *
+ *      declare f as function () returning static integer
+ *
  * i.e., the `static` has to be taken away from `int` and given to the function
  * because it's the function that's `static`, not the `int`.
  *
  * @param ast The AST to take from.
- * @return Returns said storage (and attributes) or \ref T_NONE.
+ * @return Returns said storage (and attributes) or #T_NONE.
  */
 NODISCARD
 static c_type_t c_ast_take_storage( c_ast_t *ast ) {
@@ -358,7 +381,7 @@ static c_type_t c_ast_take_storage( c_ast_t *ast ) {
 }
 
 /**
- * Only if \a ast is a \ref K_POINTER, un-pointers \a ast.
+ * Only if \a ast is a #K_POINTER, un-pointers \a ast.
  *
  * @param ast The AST to un-pointer.
  * @param rv_qual_stids If \a ast is a pointer, receives the qualifier(s) of
@@ -367,24 +390,28 @@ static c_type_t c_ast_take_storage( c_ast_t *ast ) {
  * associated with the `typedef` for the `struct` and _not_ the actual `struct`
  * the `typedef` is a `typedef` for:
  * ```
- * decl_c: {
+ * $$_ast: {
  *   sname: { string: "x", scopes: "none" },
  *   kind: { value: 0x400, string: "pointer" },
  *   ...
- *   type: { btid: 0x1, stid: 0x2, atid: 0x4, string: "none" },
- *   to_ast: {
- *     sname: { string: "" },
- *     kind: { value: 0x20, string: "typedef" },
- *     ...
- *     type: { btid: 0x20000001, stid: 0x200000002, atid: 0x4, string: "const" }.
- *     for_ast: {
- *       sname: { string: "S", scopes: "struct" },
- *       kind: { value: 0x8, string: "class, struct, or union" },
+ *   ptr_ref: {
+ *     to_ast: {
+ *       sname: null,
+ *       kind: { value: 0x20, string: "typedef" },
  *       ...
- *       type: { btid: 0x1000001, stid: 0x2, atid: 0x4, string: "struct" },
- *       csu_sname: { string: "S", scopes: "none" }
- *     },
- *     ...
+ *       type: { btid: 0x20000001, stid: 0x200000002, ..., string: "const" },
+ *       tdef: {
+ *         for_ast: {
+ *           sname: { string: "S", scopes: "struct" },
+ *           kind: { value: 0x8, string: "class, struct, or union" },
+ *           ...
+ *           type: { btid: 0x1000001, ..., string: "struct" },
+ *           csu: {
+ *             csu_sname: { string: "S", scopes: "none" }
+ *           }
+ *         }
+ *       }
+ *     }
  *   }
  * }
  * ```
@@ -416,7 +443,7 @@ static c_ast_t const* c_ast_unpointer_qual( c_ast_t const *ast,
 }
 
 /**
- * Only if \a ast is a \ref K_REFERENCE, un-references \a ast.
+ * Only if \a ast is a #K_REFERENCE, un-references \a ast.
  *
  * @param ast The AST to un-reference.
  * @param rv_qual_stids If \a ast is a reference, receives the bitwise-of of
@@ -425,24 +452,28 @@ static c_ast_t const* c_ast_unpointer_qual( c_ast_t const *ast,
  * associated with the `typedef` for the `struct` and _not_ the actual `struct`
  * the `typedef` is a `typedef` for:
  * ```
- * decl_c: {
+ * $$_ast: {
  *   sname: { string: "x", scopes: "none" },
  *   kind: { value: 0x1000, string: "reference" },
  *   ...
- *   type: { btid: 0x1, stid: 0x2, atid: 0x4, string: "none" },
- *   to_ast: {
- *     sname: { string: "" },
- *     kind: { value: 0x20, string: "typedef" },
- *     ...
- *     type: { btid: 0x20000001, stid: 0x200000002, atid: 0x4, string: "const" },
- *     for_ast: {
- *       sname: { string: "S", scopes: "struct" },
- *       kind: { value: 0x8, string: "class, struct, or union" },
+ *   ptr_ref: {
+ *     to_ast: {
+ *       sname: null,
+ *       kind: { value: 0x20, string: "typedef" },
  *       ...
- *       type: { btid: 0x1000001, stid: 0x2, atid: 0x4, string: "struct" },
- *       csu_sname: { string: "S", scopes: "none" }
- *     },
- *     ..
+ *       type: { btid: 0x20000001, stid: 0x200000002, ..., string: "const" },
+ *       tdef: {
+ *         for_ast: {
+ *           sname: { string: "S", scopes: "struct" },
+ *           kind: { value: 0x8, string: "class, struct, or union" },
+ *           ...
+ *           type: { btid: 0x1000001, ..., string: "struct" },
+ *           csu: {
+ *             csu_sname: { string: "S", scopes: "none" }
+ *           }
+ *         }
+ *       }
+ *     }
  *   }
  * }
  * ```
@@ -477,13 +508,13 @@ static c_ast_t const* c_ast_unreference_qual( c_ast_t const *ast,
  * A visitor function to find an AST node having a particular kind(s).
  *
  * @param ast The AST to check.
- * @param data The bitwise-or of the kind(s) \a ast can be.
+ * @param user_data The bitwise-or of the kind(s) \a ast can be.
  * @return Returns `true` only if the kind of \a ast is one of the kinds.
  */
 NODISCARD
-static bool c_ast_vistor_kind_any( c_ast_t *ast, user_data_t data ) {
+static bool c_ast_vistor_kind_any( c_ast_t const *ast, user_data_t user_data ) {
   assert( ast != NULL );
-  c_ast_kind_t const kinds = STATIC_CAST( c_ast_kind_t, data.ui32 );
+  c_ast_kind_t const kinds = USER_DATA_UINT_AS( c_ast_kind_t, user_data );
   return (ast->kind & kinds) != 0;
 }
 
@@ -491,13 +522,13 @@ static bool c_ast_vistor_kind_any( c_ast_t *ast, user_data_t data ) {
  * A visitor function to find an AST node having a non-empty name.
  *
  * @param ast The AST to check.
- * @param data Not used.
+ * @param user_data Not used.
  * @return Returns `true` only if \a ast has such a scoped name.
  */
 NODISCARD
-static bool c_ast_visitor_name( c_ast_t *ast, user_data_t data ) {
+static bool c_ast_visitor_name( c_ast_t const *ast, user_data_t user_data ) {
   assert( ast != NULL );
-  (void)data;
+  (void)user_data;
   return !c_sname_empty( &ast->sname );
 }
 
@@ -505,14 +536,14 @@ static bool c_ast_visitor_name( c_ast_t *ast, user_data_t data ) {
  * A visitor function to find an AST node having a particular type(s).
  *
  * @param ast The AST to check.
- * @param data A pointer to a type where each type part is the bitwise-or of
- * type IDs to find.
+ * @param user_data A pointer to a type where each type part is the bitwise-or
+ * of type IDs to find.
  * @return Returns `true` only if the type of \a ast is one of the types.
  */
 NODISCARD
-static bool c_ast_vistor_type_any( c_ast_t *ast, user_data_t data ) {
+static bool c_ast_vistor_type_any( c_ast_t const *ast, user_data_t user_data ) {
   assert( ast != NULL );
-  c_type_t const *const type = data.pc;
+  c_type_t const *const type = user_data.pc;
   return c_type_is_any( &ast->type, type );
 }
 
@@ -523,7 +554,7 @@ c_ast_t* c_ast_add_array( c_ast_t *ast, c_ast_t *array_ast, c_ast_t *of_ast ) {
   c_ast_t *const rv_ast = c_ast_add_array_impl( ast, array_ast, of_ast );
   assert( rv_ast != NULL );
   if ( c_sname_empty( &rv_ast->sname ) )
-    rv_ast->sname = c_ast_take_name( ast );
+    rv_ast->sname = c_ast_move_sname( ast );
   c_type_t const taken_type = c_ast_take_storage( array_ast->array.of_ast );
   c_type_or_eq( &array_ast->type, &taken_type );
   return rv_ast;
@@ -533,7 +564,7 @@ c_ast_t* c_ast_add_func( c_ast_t *ast, c_ast_t *func_ast, c_ast_t *ret_ast ) {
   c_ast_t *const rv_ast = c_ast_add_func_impl( ast, func_ast, ret_ast );
   assert( rv_ast != NULL );
   if ( c_sname_empty( &rv_ast->sname ) )
-    rv_ast->sname = c_ast_take_name( ast );
+    rv_ast->sname = c_ast_move_sname( ast );
   if ( func_ast->func.ret_ast != NULL ) {
     c_type_t const taken_type = c_ast_take_storage( func_ast->func.ret_ast );
     c_type_or_eq( &rv_ast->type, &taken_type );
@@ -541,38 +572,53 @@ c_ast_t* c_ast_add_func( c_ast_t *ast, c_ast_t *func_ast, c_ast_t *ret_ast ) {
   return rv_ast;
 }
 
-c_ast_t* c_ast_find_kind_any( c_ast_t *ast, c_ast_visit_dir_t dir,
-                              c_ast_kind_t kinds ) {
-  assert( kinds != 0 );
-  user_data_t const data = { .ui32 = kinds };
-  return c_ast_visit( ast, dir, c_ast_vistor_kind_any, data );
+// See comment for NONCONST_OVERLOAD regarding ().
+c_ast_t const* (c_ast_find_kind_any)( c_ast_t const *ast,
+                                      c_ast_visit_dir_t dir,
+                                      c_ast_kind_t kinds ) {
+  user_data_t ud;
+  USER_DATA_UINT( c_ast_kind_t, ud ) = kinds;
+
+  return kinds == 0 ? NULL :
+    c_ast_visit( ast, dir, &c_ast_vistor_kind_any, ud );
 }
 
-c_sname_t* c_ast_find_name( c_ast_t const *ast, c_ast_visit_dir_t dir ) {
-  user_data_t data = { .ull = 0 };
-  c_ast_t *const found_ast =
-    c_ast_visit( CONST_CAST( c_ast_t*, ast ), dir, c_ast_visitor_name, data );
+// See comment for NONCONST_OVERLOAD regarding ().
+c_sname_t const* (c_ast_find_name)( c_ast_t const *ast,
+                                    c_ast_visit_dir_t dir ) {
+  c_ast_t const *const found_ast =
+    c_ast_visit( ast, dir, &c_ast_visitor_name, USER_DATA_ZERO );
   return found_ast != NULL ? &found_ast->sname : NULL;
 }
 
-c_ast_t* c_ast_find_type_any( c_ast_t *ast, c_ast_visit_dir_t dir,
-                              c_type_t const *type ) {
-  user_data_t const data = { .pc = type };
-  return c_ast_visit( ast, dir, c_ast_vistor_type_any, data );
+c_ast_t const* c_ast_find_param_named( c_ast_t const *func_ast,
+                                       char const *name,
+                                       c_ast_t const *stop_ast ) {
+  assert( func_ast != NULL );
+  assert( is_1_bit_only_in_set( func_ast->kind, K_ANY_FUNCTION_LIKE ) );
+  assert( name != NULL );
+
+  c_sname_t const sname = C_SNAME_LIT( name );
+
+  FOREACH_AST_FUNC_PARAM( param, func_ast ) {
+    c_ast_t const *const param_ast = c_param_ast( param );
+    if ( param_ast == stop_ast )
+      break;
+    c_sname_t const *const param_sname =
+      c_ast_find_name( param_ast, C_VISIT_DOWN );
+    if ( param_sname != NULL && c_sname_cmp( param_sname, &sname ) == 0 )
+      return param_ast;
+  } // for
+
+  return NULL;
 }
 
-bool c_ast_is_bit_field_kind( c_ast_t const *ast ) {
-  ast = c_ast_untypedef( ast );
-  switch ( ast->kind ) {
-    case K_BUILTIN: {
-      c_tid_t const btids = c_tid_normalize( ast->type.btids );
-      return is_1n_bit_only_in_set( c_tid_no_tpid( btids ), TB_ANY_INTEGRAL );
-    }
-    case K_ENUM:
-      return OPT_LANG_IS( enum_BITFIELDS );
-    default:
-      return false;
-  } // switch
+// See comment for NONCONST_OVERLOAD regarding ().
+c_ast_t const* (c_ast_find_type_any)( c_ast_t const *ast, c_ast_visit_dir_t dir,
+                                      c_type_t const *type ) {
+  return c_ast_visit(
+    ast, dir, &c_ast_vistor_type_any, (user_data_t){ .pc = type }
+  );
 }
 
 bool c_ast_is_builtin_any( c_ast_t const *ast, c_tid_t btids ) {
@@ -585,6 +631,50 @@ bool c_ast_is_builtin_any( c_ast_t const *ast, c_tid_t btids ) {
   return is_1n_bit_only_in_set( c_tid_no_tpid( ast_btids ), btids );
 }
 
+bool c_ast_is_integral( c_ast_t const *ast ) {
+  ast = c_ast_untypedef( ast );
+  switch ( ast->kind ) {
+    case K_BUILTIN:
+      return c_tid_is_any( ast->type.btids, TB_ANY_INTEGRAL );
+    case K_ENUM:
+      return true;
+    default:
+      return false;
+  } // switch
+}
+
+bool c_ast_is_likely_ctor( c_ast_t const *type_ast ) {
+  assert( type_ast != NULL );
+
+  //
+  // Assume the declaration is for a constructor unless:
+  //
+  // + The current language does _not_ support constructors.
+  if ( !OPT_LANG_IS( CONSTRUCTORS ) )
+    return false;
+
+  // + The type is _not_ none (because constructors don't have return types).
+  if ( type_ast->type.btids != TB_NONE )
+    return false;
+
+  if (  // + The type does _not_ have any constructor-only storage-class-like
+        //   types (e.g., explicit).
+        !c_tid_is_any( type_ast->type.stids, TS_CONSTRUCTOR_ONLY ) &&
+        //
+        // And:
+        //
+        // + The type doed _not_ have a storage-class-like type that may be
+        //   applied to constructors.
+        !is_1n_bit_only_in_set(
+          c_tid_no_tpid( type_ast->type.stids ),
+          c_tid_no_tpid( TS_CONSTRUCTOR_DECL )
+        ) ) {
+    return false;
+  }
+
+  return true;
+}
+
 bool c_ast_is_ptr_to_kind_any( c_ast_t const *ast, c_ast_kind_t kinds ) {
   ast = c_ast_unpointer( ast );
   return ast != NULL && (ast->kind & kinds) != 0;
@@ -650,6 +740,13 @@ c_ast_t const* c_ast_is_tid_any_qual( c_ast_t const *ast, c_tid_t tids,
   return c_ast_is_tid_any_qual_impl( ast, tids, *rv_qual_stids );
 }
 
+bool c_ast_is_untyped( c_ast_t const *ast ) {
+  assert( ast != NULL );
+  return  ast->kind == K_NAME &&
+          (c_sname_empty( &ast->name.sname ) ||
+           c_sname_cmp( &ast->sname, &ast->name.sname ) == 0);
+}
+
 c_ast_t const* c_ast_leaf( c_ast_t const *ast ) {
   while ( c_ast_is_referrer( ast ) ) {
     c_ast_t const *const child_ast = ast->parent.of_ast;
@@ -660,7 +757,18 @@ c_ast_t const* c_ast_leaf( c_ast_t const *ast ) {
   return ast;
 }
 
-unsigned c_ast_oper_overload( c_ast_t const *ast ) {
+c_sname_t c_ast_move_sname( c_ast_t *ast ) {
+  assert( ast != NULL );
+  c_sname_t *const found_sname = c_ast_find_name( ast, C_VISIT_DOWN );
+  c_sname_t rv_sname;
+  if ( found_sname == NULL )
+    c_sname_init( &rv_sname );
+  else
+    rv_sname = c_sname_move( found_sname );
+  return rv_sname;
+}
+
+c_func_member_t c_ast_op_overload( c_ast_t const *ast ) {
   assert( ast != NULL );
   assert( ast->kind == K_OPERATOR );
 
@@ -668,21 +776,25 @@ unsigned c_ast_oper_overload( c_ast_t const *ast ) {
   // If the operator is either member or non-member only, then it's that.
   //
   c_operator_t const *const op = ast->oper.operator;
-  switch ( op->flags ) {
-    case C_OPER_MEMBER:
-    case C_OPER_NON_MEMBER:
-    case C_OPER_NOT_OVERLOADABLE:
-      return op->flags;
+  switch ( op->overload ) {
+    case C_OVERLOAD_NONE:
+    case C_OVERLOAD_MEMBER:
+    case C_OVERLOAD_NON_MEMBER:
+      return STATIC_CAST( c_func_member_t, op->overload );
+    case C_OVERLOAD_EITHER:
+      break;
   } // switch
 
   //
   // Otherwise, the operator can be either: see if the user specified which one
   // explicitly.
   //
-  switch ( ast->oper.flags ) {
-    case C_OPER_MEMBER:
-    case C_OPER_NON_MEMBER:
-      return ast->oper.flags;
+  switch ( ast->oper.member ) {
+    case C_FUNC_UNSPECIFIED:
+      break;
+    case C_FUNC_MEMBER:
+    case C_FUNC_NON_MEMBER:
+      return ast->oper.member;
   } // switch
 
   //
@@ -690,11 +802,11 @@ unsigned c_ast_oper_overload( c_ast_t const *ast ) {
   // has a member-only type qualifier.
   //
   if ( c_tid_is_any( ast->type.stids, TS_MEMBER_FUNC_ONLY ) )
-    return C_OPER_MEMBER;
+    return C_FUNC_MEMBER;
 
-  size_t const n_params = c_ast_params_count( ast );
+  size_t const n_params = slist_len( &ast->oper.param_ast_list );
 
-  switch ( op->oper_id ) {
+  switch ( op->op_id ) {
     case C_OP_NEW:
     case C_OP_NEW_ARRAY:
     case C_OP_DELETE:
@@ -704,11 +816,11 @@ unsigned c_ast_oper_overload( c_ast_t const *ast ) {
       // they have a name (of a class) or declared static.
       //
       return  !c_sname_empty( &ast->sname ) ||
-              c_tid_is_any( ast->type.stids, TS_STATIC ) ?
-        C_OPER_MEMBER : C_OPER_NON_MEMBER;
+              c_tid_is_any( ast->type.stids, TS_static ) ?
+        C_FUNC_MEMBER : C_FUNC_NON_MEMBER;
 
-    case C_OP_MINUS2:
-    case C_OP_PLUS2:
+    case C_OP_MINUS_MINUS:
+    case C_OP_PLUS_PLUS:
       //
       // Special case for ++ and -- operators: if the number of parameters is:
       //
@@ -718,8 +830,8 @@ unsigned c_ast_oper_overload( c_ast_t const *ast ) {
       //
       if ( n_params == 1 ) {
         c_ast_t const *const param_ast = c_param_ast( c_ast_params( ast ) );
-        return c_ast_is_builtin_any( param_ast, TB_INT ) ?
-          C_OPER_MEMBER : C_OPER_NON_MEMBER;
+        return c_ast_is_builtin_any( param_ast, TB_int ) ?
+          C_FUNC_MEMBER : C_FUNC_NON_MEMBER;
       }
       // The 0 and 2 cases are handled below.
       break;
@@ -733,14 +845,46 @@ unsigned c_ast_oper_overload( c_ast_t const *ast ) {
   // parameters given.
   //
   if ( n_params == op->params_min )
-    return C_OPER_MEMBER;
+    return C_FUNC_MEMBER;
   if ( n_params == op->params_max )
-    return C_OPER_NON_MEMBER;
+    return C_FUNC_NON_MEMBER;
 
   //
   // We can't determine which one, so give up.
   //
-  return C_OPER_UNSPECIFIED;
+  return C_FUNC_UNSPECIFIED;
+}
+
+void c_ast_op_params_min_max( c_ast_t const *ast, unsigned *params_min,
+                              unsigned *params_max ) {
+  assert( ast != NULL );
+  assert( ast->kind == K_OPERATOR );
+  assert( params_min != NULL );
+  assert( params_max != NULL );
+
+  c_operator_t const *const op = ast->oper.operator;
+  bool const is_ambiguous = c_op_is_ambiguous( op );
+  bool const is_params_unlimited = op->params_max == C_OP_PARAMS_UNLIMITED;
+
+  switch ( c_ast_op_overload( ast ) ) {
+    case C_FUNC_NON_MEMBER:
+      // Non-member operators must always take at least one parameter (the
+      // enum, class, struct, or union for which it's overloaded).
+      *params_min = is_ambiguous || is_params_unlimited ? 1 : op->params_max;
+      *params_max = op->params_max;
+      break;
+    case C_FUNC_MEMBER:
+      if ( !is_params_unlimited ) {
+        *params_min = op->params_min;
+        *params_max = is_ambiguous ? 1 : op->params_min;
+        break;
+      }
+      FALLTHROUGH;
+    case C_FUNC_UNSPECIFIED:
+      *params_min = op->params_min;
+      *params_max = op->params_max;
+      break;
+  } // switch
 }
 
 c_ast_t* c_ast_patch_placeholder( c_ast_t *type_ast, c_ast_t *decl_ast ) {
@@ -749,7 +893,7 @@ c_ast_t* c_ast_patch_placeholder( c_ast_t *type_ast, c_ast_t *decl_ast ) {
     return type_ast;
 
   if ( type_ast->parent_ast == NULL ) {
-    c_ast_t *const placeholder_ast =
+    c_ast_t const *const placeholder_ast =
       c_ast_find_kind_any( decl_ast, C_VISIT_DOWN, K_PLACEHOLDER );
     if ( placeholder_ast != NULL ) {
       if ( type_ast->depth >= decl_ast->depth ||
@@ -759,7 +903,7 @@ c_ast_t* c_ast_patch_placeholder( c_ast_t *type_ast, c_ast_t *decl_ast ) {
         // is discarded.
         //
         if ( c_sname_empty( &type_ast->sname ) )
-          type_ast->sname = c_ast_take_name( decl_ast );
+          type_ast->sname = c_ast_move_sname( decl_ast );
         return type_ast;
       }
       //
@@ -788,7 +932,7 @@ c_ast_t* c_ast_patch_placeholder( c_ast_t *type_ast, c_ast_t *decl_ast ) {
   decl_ast->type.atids |= taken_type.atids;
 
   if ( c_sname_empty( &decl_ast->sname ) )
-    decl_ast->sname = c_ast_take_name( type_ast );
+    decl_ast->sname = c_ast_move_sname( type_ast );
 
   return decl_ast;
 }
@@ -797,29 +941,31 @@ c_ast_t* c_ast_pointer( c_ast_t *ast, c_ast_list_t *ast_list ) {
   assert( ast != NULL );
   c_ast_t *const ptr_ast =
     c_ast_new( K_POINTER, ast->depth, &ast->loc, ast_list );
-  ptr_ast->sname = c_ast_take_name( ast );
+  ptr_ast->sname = c_ast_move_sname( ast );
   c_ast_set_parent( ast, ptr_ast );
   return ptr_ast;
 }
 
-c_ast_t* c_ast_root( c_ast_t *ast ) {
+// See comment for NONCONST_OVERLOAD regarding ().
+c_ast_t const* (c_ast_root)( c_ast_t const *ast ) {
   assert( ast != NULL );
   while ( ast->parent_ast != NULL )
     ast = ast->parent_ast;
   return ast;
 }
 
-c_sname_t c_ast_take_name( c_ast_t *ast ) {
+c_ast_t c_ast_sub_typedef( c_ast_t const *ast ) {
   assert( ast != NULL );
-  c_sname_t *const found_sname = c_ast_find_name( ast, C_VISIT_DOWN );
-  c_sname_t rv_sname;
-  if ( found_sname == NULL ) {
-    c_sname_init( &rv_sname );
-  } else {
-    rv_sname = *found_sname;
-    c_sname_init( found_sname );
-  }
-  return rv_sname;
+  assert( ast->kind == K_TYPEDEF );
+
+  c_tid_t qual_stids;
+  c_ast_t rv_ast = *c_ast_untypedef_qual( ast, &qual_stids );
+  rv_ast.align = ast->align;
+  rv_ast.loc = ast->loc;
+  rv_ast.type.stids |= qual_stids;
+  if ( c_ast_is_integral( &rv_ast ) )
+    rv_ast.bit_field.bit_width = ast->bit_field.bit_width;
+  return rv_ast;
 }
 
 c_type_t c_ast_take_type_any( c_ast_t *ast, c_type_t const *type ) {
@@ -833,7 +979,8 @@ c_type_t c_ast_take_type_any( c_ast_t *ast, c_type_t const *type ) {
   return rv_type;
 }
 
-c_ast_t const* c_ast_unpointer( c_ast_t const *ast ) {
+// See comment for NONCONST_OVERLOAD regarding ().
+c_ast_t const* (c_ast_unpointer)( c_ast_t const *ast ) {
   ast = c_ast_untypedef( ast );
   return ast->kind == K_POINTER ? c_ast_untypedef( ast->ptr_ref.to_ast ) : NULL;
 }
@@ -848,11 +995,12 @@ c_ast_t const* c_ast_unreference( c_ast_t const *ast ) {
   } // for
 }
 
-c_ast_t const* c_ast_unrvalue_reference( c_ast_t const *ast ) {
+// See comment for NONCONST_OVERLOAD regarding ().
+c_ast_t const* (c_ast_unreference_any)( c_ast_t const *ast ) {
   // This is a loop to implement the reference-collapsing rule.
   for (;;) {
     ast = c_ast_untypedef( ast );
-    if ( ast->kind != K_RVALUE_REFERENCE )
+    if ( (ast->kind & K_ANY_REFERENCE) == 0 )
       return ast;
     ast = ast->ptr_ref.to_ast;
   } // for
@@ -881,4 +1029,7 @@ c_ast_t const* c_ast_untypedef_qual( c_ast_t const *ast,
 }
 
 ///////////////////////////////////////////////////////////////////////////////
+
+/** @} */
+
 /* vim:set et sw=2 ts=2: */
