@@ -2,7 +2,7 @@
 **      cdecl -- C gibberish translator
 **      src/c_sname.h
 **
-**      Copyright (C) 2019-2023  Paul J. Lucas
+**      Copyright (C) 2019-2024  Paul J. Lucas
 **
 **      This program is free software: you can redistribute it and/or modify
 **      it under the terms of the GNU General Public License as published by
@@ -32,12 +32,13 @@
 #include "c_type.h"
 #include "slist.h"
 #include "types.h"
+#include "util.h"
 
 /// @cond DOXYGEN_IGNORE
 
 // standard
 #include <stdbool.h>
-#include <stddef.h>                     /* for size_t */
+#include <stddef.h>                     /* for NULL, size_t */
 
 _GL_INLINE_HEADER_BEGIN
 #ifndef C_SNAME_H_INLINE
@@ -51,11 +52,13 @@ _GL_INLINE_HEADER_BEGIN
  * Functions for dealing with "sname" (C++ scoped name) objects, e.g.,
  * `S::T::x`.
  *
- * + An sname also has a type for each scope, one of of #TB_CLASS,
- *   #TB_NAMESPACE (and possibly #TS_INLINE), #TB_SCOPE, #TB_STRUCT, or
- *   #TB_UNION.
+ * + An sname also has a type for each scope, one of of #TB_class,
+ *   #TB_namespace (and possibly #TS_inline), #TB_SCOPE, #TB_struct, or
+ *   #TB_union.
+ *
  * + The "local" of an sname is the innermost scope, e.g., `x`.  A non-empty
  *   sname always has a local.
+ *
  * + The "scope" of an sname is all but the innermost scope, e.g., `S::T`.  A
  *   non-empty sname may or may not have a scope.
  *
@@ -66,63 +69,94 @@ _GL_INLINE_HEADER_BEGIN
  */
 
 /**
- * Creates a scoped name variable \a VAR on the stack having a local \a NAME.
+ * Gets the \ref c_scope_data associated with \a SCOPE.
  *
- * @param VAR The name for the \ref c_sname_t variable.
- * @param NAME The name.
+ * @param SCOPE The scope to get the data of.  Must not be NULL.
+ * @return Returns said data.
+ *
+ * @sa c_sname_global_data()
+ * @sa c_sname_local_data()
  */
-#define SNAME_VAR_INIT(VAR,NAME) \
-  SLIST_VAR_INIT( VAR, (&(c_scope_data_t){ (NAME), T_NONE }) )
+#define c_scope_data(SCOPE)       POINTER_CAST( c_scope_data_t*, (SCOPE)->data )
 
 /**
- * Gets the data associated with \a SCOPE.
+ * Gets the global scope data of \a SNAME (which is the data of the outermost
+ * scope).
  *
- * @param SCOPE The scope to get the data of.
+ * @param SNAME The scoped name to get the global scope data of.  Must not be
+ * NULL.
+ * @return Returns the global scope data of \a SNAME.
  *
- * @note This is a macro instead of an inline function so it'll work with
- * either a `const` or non-`const` \a SCOPE.
+ * @sa c_scope_data()
+ * @sa c_sname_global_type()
+ * @sa c_sname_local_data()
  */
-#define c_scope_data(SCOPE) \
-  POINTER_CAST( c_scope_data_t*, (SCOPE)->data )
+#define c_sname_global_data(SNAME) \
+  c_scope_data( (SNAME)->head )
+
+/**
+ * Creates a scoped name literal with a local name of \a NAME.
+ *
+ * @param NAME The local name.
+ * @return Returns said scoped name.
+ *
+ * @warning c_sname_cleanup() must _not_ be called on the returned value.
+ */
+#define C_SNAME_LIT(NAME) \
+  SLIST_LIT( (&(c_scope_data_t){ CONST_CAST( char*, (NAME) ), T_NONE }) )
+
+/**
+ * Gets the local scope data of \a SNAME (which is the data of the innermost
+ * scope).
+ *
+ * @param SNAME The scoped name to get the local scope data of.  Must not be
+ * NULL.
+ * @return Returns the local scope data of \a SNAME.
+ *
+ * @sa c_scope_data()
+ * @sa c_sname_local_type()
+ * @sa c_sname_global_data()
+ */
+#define c_sname_local_data(SNAME) c_scope_data( (SNAME)->tail )
 
 /**
  * Convenience macro for iterating over all scopes of an sname.
  *
- * @param VAR The \ref c_scope_t loop variable.
+ * @param VAR The \ref slist_node loop variable.
  * @param SNAME The \ref c_sname_t to iterate over the scopes of.
  *
  * @sa #FOREACH_SNAME_SCOPE_UNTIL()
  */
 #define FOREACH_SNAME_SCOPE(VAR,SNAME) \
-  FOREACH_SLIST_NODE( VAR, SNAME )
+  FOREACH_SLIST_NODE( VAR, (SNAME) )
 
 /**
  * Convenience macro for iterating over all scopes of an sname up to but not
  * including \a END.
  *
- * @param VAR The \ref c_scope_t loop variable.
+ * @param VAR The \ref slist_node loop variable.
  * @param SNAME The \ref c_sname_t to iterate over the scopes of.
  * @param END The scope to end before; may be NULL.
  *
  * @sa #FOREACH_SNAME_SCOPE()
  */
 #define FOREACH_SNAME_SCOPE_UNTIL(VAR,SNAME,END) \
-  FOREACH_SLIST_NODE_UNTIL( VAR, SNAME, END )
+  FOREACH_SLIST_NODE_UNTIL( VAR, (SNAME), (END) )
 
 ///////////////////////////////////////////////////////////////////////////////
 
 /**
- * Data for each scope of an sname.
+ * Data for each scope of an \ref c_sname_t.
  */
 struct c_scope_data {
   /**
    * The scope's name.
    */
-  char const *name;
+  char *name;
 
   /**
-   * The scope's type, one of: #TB_CLASS, #TB_STRUCT, #TB_UNION, [#TS_INLINE]
-   * #TB_NAMESPACE, or #TB_SCOPE.
+   * The scope's type, one of: #TB_class, #TB_struct, #TB_union, [#TS_inline]
+   * #TB_namespace, or #TB_SCOPE.
    */
   c_type_t type;
 };
@@ -145,9 +179,9 @@ int c_scope_data_cmp( c_scope_data_t const *i_data,
 /**
  * Duplicates \a data.
  *
- * @param data The \ref c_scope_data to duplicate.
- * @return Returns a duplicate of \a data.  The caller is responsible for
- * calling c_scope_data_free() on it.
+ * @param data The \ref c_scope_data to duplicate; may be NULL.
+ * @return Returns a duplicate of \a data or NULL only if \a data is NULL.  The
+ * caller is responsible for calling c_scope_data_free() on it.
  *
  * @sa c_scope_data_free()
  */
@@ -171,7 +205,7 @@ void c_scope_data_free( c_scope_data_t *data );
  * @sa c_sname_prepend_sname()
  * @sa c_sname_set()
  */
-void c_sname_append_name( c_sname_t *sname, char const *name );
+void c_sname_append_name( c_sname_t *sname, char *name );
 
 /**
  * Appends \a src onto the end of \a dst.
@@ -221,7 +255,9 @@ void c_sname_cleanup( c_sname_t *sname );
  */
 NODISCARD C_SNAME_H_INLINE
 int c_sname_cmp( c_sname_t const *i_sname, c_sname_t const *j_sname ) {
-  return slist_cmp( i_sname, j_sname, (slist_cmp_fn_t)&c_scope_data_cmp );
+  return slist_cmp(
+    i_sname, j_sname, POINTER_CAST( slist_cmp_fn_t, &c_scope_data_cmp )
+  );
 }
 
 /**
@@ -242,12 +278,15 @@ size_t c_sname_count( c_sname_t const *sname ) {
  * Duplicates \a sname.  The caller is responsible for calling
  * c_sname_cleanup() on the duplicate.
  *
- * @param sname The scoped name to duplicate.
- * @return Returns a duplicate of \a sname.
+ * @param sname The scoped name to duplicate; may be NULL.
+ * @return Returns a duplicate of \a sname or an empty scoped name if \a sname
+ * is NULL.
  */
 NODISCARD C_SNAME_H_INLINE
 c_sname_t c_sname_dup( c_sname_t const *sname ) {
-  return slist_dup( sname, -1, (slist_dup_fn_t)&c_scope_data_dup );
+  return slist_dup(
+    sname, -1, POINTER_CAST( slist_dup_fn_t, &c_scope_data_dup )
+  );
 }
 
 /**
@@ -261,34 +300,6 @@ bool c_sname_empty( c_sname_t const *sname ) {
   return slist_empty( sname );
 }
 
-/**
- * If the local scope-type of \a sname is #TB_NAMESPACE, make all scope-types
- * of all enclosing scopes that are either #TB_NONE or #TB_SCOPE also be
- * #TB_NAMESPACE since a namespace can only nest within another namespace.
- *
- * @param sname The scoped name to fill in namespaces.
- *
- * @note If there are scope-types that are something other than either #TB_NONE
- * or #TB_SCOPE, this is an error and will be caught by c_sname_check().
- */
-void c_sname_fill_in_namespaces( c_sname_t *sname );
-
-/**
- * Gets the first scope-type of \a sname (which is the type of the outermost
- * scope).
- *
- * @param sname The scoped name to get the first scope-type of.
- * @return Returns the first scope-type or #T_NONE if \a sname is empty.
- *
- * @sa c_sname_local_type()
- * @sa c_sname_scope_type()
- * @sa c_sname_set_first_type()
- */
-NODISCARD C_SNAME_H_INLINE
-c_type_t const* c_sname_first_type( c_sname_t const *sname ) {
-  return c_sname_empty( sname ) ? &T_NONE : &c_scope_data( sname->head )->type;
-}
-
 /**
  * Frees all memory associated with \a sname _including_ \a sname itself.
  *
@@ -302,21 +313,36 @@ c_type_t const* c_sname_first_type( c_sname_t const *sname ) {
 void c_sname_free( c_sname_t *sname );
 
 /**
- * Gets the fully scoped name of \a sname.
+ * Gets the global name of \a sname (which is the name of the first scope), for
+ * example the global name of `S::T::x` is `S`.
  *
- * @param sname The scoped name to get the full name of.  May be NULL.
+ * @param sname The scoped name to get the global name of; may be NULL.
  * @return Returns said name or the empty string if \a sname is empty or NULL.
  *
- * @warning The pointer returned is to a static buffer, so you can't do
- * something like call this twice in the same `printf()` statement.
- *
- * @sa c_sname_english()
+ * @sa c_sname_gibberish()
  * @sa c_sname_local_name()
  * @sa c_sname_name_atr()
- * @sa c_sname_scope_name()
+ * @sa c_sname_scope_gibberish()
+ * @sa c_sname_scope_sname()
  */
 NODISCARD
-char const* c_sname_full_name( c_sname_t const *sname );
+char const* c_sname_global_name( c_sname_t const *sname );
+
+/**
+ * Gets the global scope-type of \a sname (which is the type of the outermost
+ * scope).
+ *
+ * @param sname The scoped name to get the global scope-type of.
+ * @return Returns the global scope-type or #T_NONE if \a sname is empty.
+ *
+ * @sa c_sname_global_data()
+ * @sa c_sname_local_type()
+ * @sa c_sname_scope_type()
+ */
+NODISCARD C_SNAME_H_INLINE
+c_type_t const* c_sname_global_type( c_sname_t const *sname ) {
+  return c_sname_empty( sname ) ? &T_NONE : &c_sname_global_data( sname )->type;
+}
 
 /**
  * Initializes \a sname.
@@ -346,7 +372,7 @@ void c_sname_init( c_sname_t *sname ) {
  * @sa c_sname_init()
  */
 C_SNAME_H_INLINE
-void c_sname_init_name( c_sname_t *sname, char const *name ) {
+void c_sname_init_name( c_sname_t *sname, char *name ) {
   slist_init( sname );
   c_sname_append_name( sname, name );
 }
@@ -365,6 +391,15 @@ void c_sname_init_name( c_sname_t *sname, char const *name ) {
 NODISCARD
 bool c_sname_is_ctor( c_sname_t const *sname );
 
+/**
+ * Gets whether \a sname is an `inline` nested `namespace`.
+ *
+ * @param sname The scoped name to check.
+ * @return Returns `true` only if \a sname is an `inline` nested `namespace`.
+ */
+NODISCARD
+bool c_sname_is_inline_nested_namespace( c_sname_t const *sname );
+
 /**
  * Cleans-up all memory associated with \a list but does _not_ free \a list
  * itself.
@@ -381,20 +416,17 @@ void c_sname_list_cleanup( slist_t *list );
  * Gets the local name of \a sname (which is the name of the last scope), for
  * example the local name of `S::T::x` is `x`.
  *
- * @param sname The scoped name to get the local name of.  May be NULL.
+ * @param sname The scoped name to get the local name of; may be NULL.
  * @return Returns said name or the empty string if \a sname is empty or NULL.
  *
- * @sa c_sname_full_name()
+ * @sa c_sname_gibberish()
+ * @sa c_sname_global_name()
  * @sa c_sname_name_atr()
- * @sa c_sname_scope_name()
+ * @sa c_sname_scope_gibberish()
+ * @sa c_sname_scope_sname()
  */
-NODISCARD C_SNAME_H_INLINE
-char const* c_sname_local_name( c_sname_t const *sname ) {
-  if ( sname == NULL )
-    return "";
-  c_scope_data_t const *const local_data = slist_back( sname );
-  return local_data == NULL ? "" : local_data->name;
-}
+NODISCARD
+char const* c_sname_local_name( c_sname_t const *sname );
 
 /**
  * Gets the local scope-type of \a sname (which is the type of the innermost
@@ -403,26 +435,28 @@ char const* c_sname_local_name( c_sname_t const *sname ) {
  * @param sname The scoped name to get the local scope-type of.
  * @return Returns the local scope-type or #T_NONE if \a sname is empty.
  *
- * @sa c_sname_first_type()
+ * @sa c_sname_global_type()
+ * @sa c_sname_local_data()
  * @sa c_sname_scope_type()
- * @sa c_sname_set_local_type()
  */
 NODISCARD C_SNAME_H_INLINE
 c_type_t const* c_sname_local_type( c_sname_t const *sname ) {
-  return c_sname_empty( sname ) ? &T_NONE : &c_scope_data( sname->tail )->type;
+  return c_sname_empty( sname ) ? &T_NONE : &c_sname_local_data( sname )->type;
 }
 
-
 /**
  * Checks whether \a sname matches \a sglob where \a sglob is glob-like in that
- * `*` matches zero or more characters.  However, `*` matches only within a
+ * `*` matches zero or more characters; however, `*` matches only within a
  * single scope.  Examples:
  *
  *  + `foo*` matches all names starting with `foo` in the global scope.
+ *
  *  + `s::&zwj;*foo` matches all names ending with `foo` only within the top-
  *    level scope `s`.
+ *
  *  + `s*::&zwj;foo` matches all names equal to `foo` in all top-level scopes
  *    starting with `s`.
+ *
  *  + `s::*::&zwj;foo` matches all names equal to `foo` in any scope within the
  *    top-level scope `s`.
  *
@@ -440,7 +474,8 @@ bool c_sname_match( c_sname_t const *sname, c_sglob_t const *sglob );
 /**
  * Reinitializes \a sname and returns its former value so that it can be
  * "moved" into another scoped name via assignment.  For example:
- * ```
+ *
+ * ```c
  * c_sname_t new_sname = c_sname_move( old_sname );
  * ```
  *
@@ -469,8 +504,11 @@ c_sname_t c_sname_move( c_sname_t *sname ) {
  * @return Returns the name at \a roffset or the empty string if \a roffset
  * &ge; c_sname_count().
  *
- * @sa c_sname_full_name()
- * @sa c_sname_scope_name()
+ * @sa c_sname_gibberish()
+ * @sa c_sname_global_name()
+ * @sa c_sname_local_name()
+ * @sa c_sname_scope_gibberish()
+ * @sa c_sname_scope_sname()
  */
 NODISCARD C_SNAME_H_INLINE
 char const* c_sname_name_atr( c_sname_t const *sname, size_t roffset ) {
@@ -519,33 +557,33 @@ void c_sname_prepend_sname( c_sname_t *dst, c_sname_t *src ) {
 }
 
 /**
- * Gets just the scope name of \a sname.
+ * Gets just the scope sname of \a sname.
  * Examples:
  *  + For `a::b::c`, returns `a::b`.
- *  + For `c`, returns the empty string.
+ *  + For `c`, returns an empty scoped name.
  *
- * @param sname The scoped name to get the scope name of.  May be NULL.
- * @return Returns said name or the empty string if \a sname is empty, NULL, or
- * not within a scope.
+ * @param sname The scoped name to get the scope name of; may be NULL.
+ * @return Returns said scoped name or an empty scoped name if \a sname is
+ * empty, NULL, or not within a scope.
  *
- * @warning The pointer returned is to a static buffer, so you can't do
- * something like call this twice in the same `printf()` statement.
- *
- * @sa c_sname_full_name()
+ * @sa c_sname_gibberish()
+ * @sa c_sname_global_name()
  * @sa c_sname_local_name()
  * @sa c_sname_name_atr()
+ * @sa c_sname_scope_gibberish()
  */
 NODISCARD
-char const* c_sname_scope_name( c_sname_t const *sname );
+c_sname_t c_sname_scope_sname( c_sname_t const *sname );
 
 /**
  * Gets the scope scope-type of \a sname (which is the type of the next
  * innermost scope).
  *
  * @param sname The scoped name to get the scope scope-type of.
- * @return Returns the scope-type #T_NONE if \a sname is empty or not within a
- * scope.
+ * @return Returns the scope scope-type or \a sname or #T_NONE if \a sname is
+ * empty or not within a scope.
  *
+ * @sa c_sname_global_type()
  * @sa c_sname_local_type()
  * @sa c_sname_set_scope_type()
  */
@@ -561,48 +599,32 @@ c_type_t const* c_sname_scope_type( c_sname_t const *sname ) {
  * @param dst_sname The scoped name to set.
  * @param src_sname The scoped name to set \a dst_sname to. Ownership is taken.
  *
+ * @note If \a dst_sname `==` \a src_name, does nothing.
+ *
  * @sa c_sname_append_name()
  * @sa c_sname_append_sname()
  * @sa c_sname_move()
  * @sa c_sname_prepend_sname()
  */
-C_SNAME_H_INLINE
-void c_sname_set( c_sname_t *dst_sname, c_sname_t *src_sname ) {
-  c_sname_cleanup( dst_sname );
-  c_sname_append_sname( dst_sname, src_sname );
-}
+void c_sname_set( c_sname_t *dst_sname, c_sname_t *src_sname );
 
 /**
- * Sets the first scope-type of \a sname (which is the type of the outermost
- * scope).
+ * Sets all the scope-types (except that of the local scope) of \a sname to the
+ * types found in `typedef`'s names.
  *
- * @param sname The scoped name to set the first scope-type of.
- * @param type The type.
+ * For example, given:
  *
- * @sa c_sname_first_type()
- * @sa c_sname_set_local_type()
- * @sa c_sname_set_scope_type()
+ *      class C { struct S; };
+ *
+ * and an \a sname of `C::S::x`, set `C`'s scope-type to #TB_class and `S`'s
+ * scope-type to #TB_struct; the scope-type of `x` is not changed.
+ *
+ * If there is no `typedef` for a partial scoped name and the scope-type is
+ * either #TB_SCOPE or not set, then sets that scope-type to #TB_namespace.
+ *
+ * @param sname The scoped name to set all the scope-types of.
  */
-C_SNAME_H_INLINE
-void c_sname_set_first_type( c_sname_t *sname, c_type_t const *type ) {
-  c_scope_data( sname->head )->type = *type;
-}
-
-/**
- * Sets the local scope-type of \a sname (which is the type of the innermost
- * scope).
- *
- * @param sname The scoped name to set the local scope-type of.
- * @param type The type.
- *
- * @sa c_sname_local_type()
- * @sa c_sname_set_first_type()
- * @sa c_sname_set_scope_type()
- */
-C_SNAME_H_INLINE
-void c_sname_set_local_type( c_sname_t *sname, c_type_t const *type ) {
-  c_scope_data( sname->tail )->type = *type;
-}
+void c_sname_set_all_types( c_sname_t *sname );
 
 /**
  * Sets the scope scope-type of \a sname (which is the type of the next
@@ -612,8 +634,6 @@ void c_sname_set_local_type( c_sname_t *sname, c_type_t const *type ) {
  * @param type The type.
  *
  * @sa c_sname_scope_type()
- * @sa c_sname_set_first_type()
- * @sa c_sname_set_local_type()
  */
 C_SNAME_H_INLINE
 void c_sname_set_scope_type( c_sname_t *sname, c_type_t const *type ) {
@@ -622,6 +642,16 @@ void c_sname_set_scope_type( c_sname_t *sname, c_type_t const *type ) {
     data->type = *type;
 }
 
+/**
+ * Checks a scoped name for warnings.
+ *
+ * @param sname The scoped name to check.
+ * @param sname_loc The location of \a sname.
+ *
+ * @sa c_name_error()
+ */
+void c_sname_warn( c_sname_t const *sname, c_loc_t const *sname_loc );
+
 ///////////////////////////////////////////////////////////////////////////////
 
 /** @} */
