@@ -2,7 +2,7 @@
 **      cdecl -- C gibberish translator
 **      src/c_typedef.c
 **
-**      Copyright (C) 2017-2023  Paul J. Lucas
+**      Copyright (C) 2017-2024  Paul J. Lucas
 **
 **      This program is free software: you can redistribute it and/or modify
 **      it under the terms of the GNU General Public License as published by
@@ -27,17 +27,20 @@
 // local
 #include "pjl_config.h"                 /* must go first */
 #include "c_typedef.h"
-#include "cdecl.h"
 #include "c_ast.h"
 #include "c_lang.h"
-#include "gibberish.h"
+#include "cdecl.h"
+#include "lexer.h"
 #include "options.h"
+#include "parse.h"
+#include "types.h"
 #include "util.h"
 
 /// @cond DOXYGEN_IGNORE
 
 // standard
 #include <assert.h>
+#include <stddef.h>                     /* for NULL */
 #include <stdlib.h>
 #include <sysexits.h>
 
@@ -48,28 +51,20 @@
  * @{
  */
 
-/**
- * Convenience macro for specifying a \ref c_typedef literal with an AST having
- * \a SNAME.
- *
- * @param SNAME The sname.
- */
-#define C_TYPEDEF_SNAME_LIT(SNAME) \
-  C_TYPEDEF_AST_LIT( &(c_ast_t const){ .sname = (SNAME) } )
-
 /**
  * Helper macro for adding a \ref predef_type to an array of them.  It includes
  * the source line number it's defined on.
  *
  * @param S The string literal of the **cdecl** command defining a type.
  */
-#define PT(S)                     { S, __LINE__ }
+#define PT(S)                     { (S), __LINE__ }
 
 ///////////////////////////////////////////////////////////////////////////////
 
 /**
  * Contains a **cdecl** command defining a type and the source line number it's
- * defined on.
+ * defined on.  The line number is included in an error message if the type
+ * fails to parse.
  */
 struct predef_type {
   char const *str;                      ///< **Cdecl** command defining a type.
@@ -114,6 +109,7 @@ static predef_type_t const PREDEFINED_KNR_C[] = {
   PT( "typedef          int   jmp_buf[37]" ),
   PT( "typedef          long  off_t" ),
   PT( "typedef          long  time_t" ),
+  PT( "        struct         tm" ),
 
   PT( NULL )
 };
@@ -374,6 +370,7 @@ static predef_type_t const PREDEFINED_STD_CPP[] = {
   PT( "namespace std { class ctype_base { using mask = unsigned; }; }" ),
   PT( "namespace std { class                    messages_base; }" ),
   PT( "namespace std { class                    money_base; }" ),
+  PT( "namespace std                    { using new_handler = void (*)(); }" ),
   PT( "namespace std { struct                   nothrow_t; }" ),
   PT( "namespace std { class ios_base   { using openmode = unsigned; }; }" ),
   PT( "namespace std { class                    out_of_range; }" ),
@@ -415,6 +412,7 @@ static predef_type_t const PREDEFINED_STD_CPP[] = {
   PT( "namespace std { class                   osyncstream; }" ),
   PT( "namespace std { class                  wosyncstream; }" ),
   PT( "namespace std { class                    time_base; }" ),
+  PT( "namespace std { struct                   tm; }" ),
   PT( "namespace std { class                    type_info; }" ),
   PT( "namespace std { class                    underflow_error; }" ),
 
@@ -429,6 +427,7 @@ static predef_type_t const PREDEFINED_STD_CPP[] = {
  */
 static predef_type_t const PREDEFINED_STD_CPP_11[] = {
   PT( "namespace std           {    struct      adopt_lock_t; }" ),
+  PT( "namespace std           {    struct      allocator_arg_t; }" ),
   //
   // These atomic_* types are supposed to be typedefs, but they're typedefs to
   // instantiated templates and cdecl doesn't support templates, so we make
@@ -496,6 +495,7 @@ static predef_type_t const PREDEFINED_STD_CPP_11[] = {
   PT( "namespace std          {      class      error_category; }" ),
   PT( "namespace std          {      class      error_code; }" ),
   PT( "namespace std          {      class      error_condition; }" ),
+  PT( "namespace std::regex_constants { using   error_type = unsigned; }" ),
   PT( "namespace std          { class ios_base::failure; }" ),
   PT( "namespace std          { enum class      future_errc; }" ),
   PT( "namespace std          {      class      future_error; }" ),
@@ -533,6 +533,7 @@ static predef_type_t const PREDEFINED_STD_CPP_11[] = {
  */
 static predef_type_t const PREDEFINED_STD_CPP_17[] = {
   PT( "namespace std             { enum class     align_val_t; }" ),
+  PT( "namespace std             {     struct     any; }" ),
   PT( "namespace std             {      class     bad_any_cast; }" ),
   PT( "namespace std             {      class     bad_optional_access; }" ),
   PT( "namespace std             {      class     bad_variant_access; }" ),
@@ -545,33 +546,47 @@ static predef_type_t const PREDEFINED_STD_CPP_17[] = {
   PT( "namespace std::filesystem {      class     file_status; }" ),
   PT( "namespace std::filesystem { enum class     file_type; }" ),
   PT( "namespace std::filesystem {      class     filesystem_error; }" ),
+  PT( "namespace std             {     struct     from_chars_result; }" ),
+  PT( "namespace std::pmr        {      class     memory_resource; }" ),
+  PT( "namespace std             {     struct     mono_state; }" ),
+  PT( "namespace std::pmr        {      class     monotonic_buffer_resource; }" ),
   PT( "namespace std::filesystem {      class     path; }" ),
   PT( "namespace std::filesystem { enum class     perms; }" ),
   PT( "namespace std::filesystem { enum class     perm_options; }" ),
+  PT( "namespace std::pmr        {     struct     pool_options; }" ),
   PT( "namespace std::filesystem {      class     recursive_directory_iterator; }" ),
   PT( "namespace std::filesystem {     struct     space_info; }" ),
   PT( "namespace std             {      class     string_view; }" ),
   PT( "namespace std             {      class  u16string_view; }" ),
   PT( "namespace std             {      class  u32string_view; }" ),
   PT( "namespace std             {      class    wstring_view; }" ),
+  PT( "namespace std::pmr        {      class     synchronized_pool_resource; }" ),
+  PT( "namespace std             {     struct     time_spec; }" ),
+  PT( "namespace std             {     struct     to_chars_result; }" ),
+  PT( "namespace std::pmr        {      class     unsynchronized_pool_resource; }" ),
 
   PT( NULL )
 };
 
 /**
  * Predefined types for C++20.
+ *
+ * @sa PREDEFINED_STD_CPP_20_REQUIRED
  */
 static predef_type_t const PREDEFINED_STD_CPP_20[] = {
   PT( "namespace std         {      class   ambiguous_local_time; }" ),
   PT( "namespace std::chrono { enum class   choose; }" ),
+  PT( "namespace std         {     struct   compare_three_way; }" ),
   PT( "namespace std::chrono {      class   day; }" ),
   PT( "namespace std         {     struct   destroying_delete_t; }" ),
+  PT( "namespace std         { enum class   endian; }" ),
   PT( "namespace std::chrono {     struct   file_clock; }" ),
   PT( "namespace std         {      class   format_error; }" ),
   PT( "namespace std::chrono {     struct   gps_clock; }" ),
   PT( "namespace std::chrono {     struct   is_clock; }" ),
   PT( "namespace std         {      class   jthread; }" ),
   PT( "namespace std::chrono {     struct   last_spec; }" ),
+  PT( "namespace std         {      class   latch; }" ),
   PT( "namespace std::chrono {      class   leap_second; }" ),
   PT( "namespace std::chrono {     struct   local_info; }" ),
   PT( "namespace std::chrono {     struct   local_t; }" ),
@@ -582,11 +597,14 @@ static predef_type_t const PREDEFINED_STD_CPP_20[] = {
   PT( "namespace std::chrono {      class   month_weekday_last; }" ),
   PT( "namespace std::chrono {      class   nonexistent_local_time; }" ),
   PT( "namespace std         {     struct   nonstopstate_t; }" ),
+  PT( "namespace std         {     struct   noop_coroutine_promise; }" ),
   PT( "namespace std         {     struct   source_location; }" ),
   PT( "namespace std         {      class u8string_view; }" ),
   PT( "namespace std         {      class   stop_source; }" ),
   PT( "namespace std         {      class   stop_token; }" ),
   PT( "namespace std         {     struct   strong_equality; }" ),
+  PT( "namespace std         {     struct   suspend_always; }" ),
+  PT( "namespace std         {     struct   suspend_never; }" ),
   PT( "namespace std::chrono {     struct   sys_info; }" ),
   PT( "namespace std::chrono {     struct   tai_clock; }" ),
   PT( "namespace std::chrono {     struct   time_zone; }" ),
@@ -610,6 +628,8 @@ static predef_type_t const PREDEFINED_STD_CPP_20[] = {
 
 /**
  * Predefined required types for C++20.
+ *
+ * @sa PREDEFINED_STD_CPP_20
  */
 static predef_type_t const PREDEFINED_STD_CPP_20_REQUIRED[] = {
   // required for operator <=>
@@ -624,22 +644,23 @@ static predef_type_t const PREDEFINED_STD_CPP_20_REQUIRED[] = {
  * Predefined types for C++23.
  */
 static predef_type_t const PREDEFINED_STD_CPP_23[] = {
-  PT( "namespace std {   using  float16_t = float; }" ),
-  PT( "namespace std {   using bfloat16_t = float; }" ),
-  PT( "namespace std {   using  float32_t = float; }" ),
-  PT( "namespace std {   using  float64_t = double; }" ),
-  PT( "namespace std {   using  float128_t = double[2]; }" ),
-  PT( "namespace std { class    spanbuf; }" ),
-  PT( "namespace std { class   wspanbuf; }" ),
-  PT( "namespace std { class    spanstream; }" ),
-  PT( "namespace std { class   ispanstream; }" ),
-  PT( "namespace std { class  wispanstream; }" ),
-  PT( "namespace std { class   ospanstream; }" ),
-  PT( "namespace std { class  wospanstream; }" ),
-  PT( "namespace std { class   wspanstream; }" ),
-  PT( "namespace std { class    stacktrace; }" ),
-  PT( "namespace std { class    stacktrace_entry; }" ),
-  PT( "namespace std { struct   unexpect_t; }" ),
+  PT( "namespace std {      using   float16_t = float; }" ),
+  PT( "namespace std {      using  bfloat16_t = float; }" ),
+  PT( "namespace std {      using   float32_t = float; }" ),
+  PT( "namespace std {      using   float64_t = double; }" ),
+  PT( "namespace std {      using   float128_t = double[2]; }" ),
+  PT( "namespace std { enum class   range_format; }" ),
+  PT( "namespace std {      class   spanbuf; }" ),
+  PT( "namespace std {      class  wspanbuf; }" ),
+  PT( "namespace std {      class   spanstream; }" ),
+  PT( "namespace std {      class  ispanstream; }" ),
+  PT( "namespace std {      class wispanstream; }" ),
+  PT( "namespace std {      class  ospanstream; }" ),
+  PT( "namespace std {      class wospanstream; }" ),
+  PT( "namespace std {      class  wspanstream; }" ),
+  PT( "namespace std {      class   stacktrace; }" ),
+  PT( "namespace std {      class   stacktrace_entry; }" ),
+  PT( "namespace std {     struct   unexpect_t; }" ),
 
   PT( NULL )
 };
@@ -677,6 +698,7 @@ static predef_type_t const PREDEFINED_GNU_C[] = {
   PT( "typedef _Float16    __fp16" ),
   PT( "typedef long double __ibm128" ),
   PT( "typedef _Float64x   __float80" ),
+
   //
   // In GNU C, this is a distinct type, not a typedef, which means you can add
   // type modifiers:
@@ -687,7 +709,7 @@ static predef_type_t const PREDEFINED_GNU_C[] = {
   //
   // To make it a distinct type in cdecl also, there would need to be a
   // distinct literal, token, and type.  The type has to be distinct in order
-  // to be round-trippable with pseudo-English.  If it reused TB_LONG_LONG,
+  // to be round-trippable with pseudo-English.  If it reused TB_long_long,
   // then you'd get:
   //
   //      cdecl> declare x as __int128
@@ -964,28 +986,14 @@ static predef_type_t const PREDEFINED_WIN32[] = {
 
 ////////// local functions ////////////////////////////////////////////////////
 
-/**
- * Cleans up \ref c_typedef data.
- *
- * @sa c_typedef_init()
- */
-static void c_typedef_cleanup( void ) {
-  // There is no c_typedef_free() function because c_typedef_add() adds only
-  // c_typedef_t nodes pointing to pre-existing AST nodes.  The AST nodes are
-  // freed independently in parser_cleanup().  Hence, this function frees only
-  // the red-black tree, its nodes, and the c_typedef_t data each node points
-  // to, but not the AST nodes the c_typedef_t data points to.
-  rb_tree_cleanup( &typedef_set, &free );
-}
-
 /**
  * Comparison function for two \ref c_typedef.
  *
  * @param i_tdef A pointer to the first \ref c_typedef.
  * @param j_tdef A pointer to the second \ref c_typedef.
  * @return Returns an integer less than, equal to, or greater than 0, according
- * to whether the `typedef` name pointed to by \a i_data is less than, equal
- * to, or greater than the `typedef` name pointed to by \a j_data.
+ * to whether the `typedef` name pointed to by \a i_tdef is less than, equal
+ * to, or greater than the `typedef` name pointed to by \a j_tdef.
  */
 NODISCARD
 static int c_typedef_cmp( c_typedef_t const *i_tdef,
@@ -999,33 +1007,51 @@ static int c_typedef_cmp( c_typedef_t const *i_tdef,
  * Creates a new \ref c_typedef.
  *
  * @param ast The AST of the type.
- * @param gib_flags The gibberish flags to use; must only be one of
- * #C_GIB_NONE, #C_GIB_TYPEDEF, or #C_GIB_USING.
+ * @param decl_flags The declaration flags to use; must only be one of
+ * #C_ENG_DECL, #C_GIB_TYPEDEF, or #C_GIB_USING.
  * @return Returns said \ref c_typedef.
  */
 NODISCARD
-static c_typedef_t* c_typedef_new( c_ast_t const *ast, unsigned gib_flags ) {
+static c_typedef_t* c_typedef_new( c_ast_t const *ast,
+                                   decl_flags_t decl_flags ) {
   assert( ast != NULL );
-  assert( is_01_bit_only_in_set( gib_flags, C_GIB_TYPEDEF | C_GIB_USING ) );
+  assert( is_1_bit_only_in_set( decl_flags, C_TYPE_DECL_ANY ) );
 
   bool const is_predefined = predef_lang_ids != LANG_NONE;
 
   c_typedef_t *const tdef = MALLOC( c_typedef_t, 1 );
   *tdef = (c_typedef_t){
     .ast = ast,
-    .gib_flags = gib_flags,
+    .decl_flags = decl_flags,
     .is_predefined = is_predefined,
     //
     // If predef_lang_ids is set, we're predefining a type that's available
     // only in those language(s); otherwise we're defining a user-defined type
     // that's available in the current language and newer.
     //
-    .lang_ids = is_predefined ? predef_lang_ids : c_lang_and_newer( opt_lang )
+    .lang_ids = is_predefined ?
+      predef_lang_ids : c_lang_and_newer( opt_lang_id )
   };
 
   return tdef;
 }
 
+/**
+ * Cleans up \ref c_typedef data.
+ *
+ * @sa c_typedefs_init()
+ */
+static void c_typedefs_cleanup( void ) {
+  //
+  // There is no c_typedef_free() function because c_typedef_add() adds only
+  // c_typedef_t nodes pointing to pre-existing AST nodes.  The AST nodes are
+  // freed independently in parser_cleanup().  Hence, this function frees only
+  // the red-black tree, its nodes, and the c_typedef_t data each node points
+  // to, but not the AST nodes the c_typedef_t data points to.
+  //
+  rb_tree_cleanup( &typedef_set, &free );
+}
+
 /**
  * Parses an array of predefined type declarations.
  *
@@ -1033,10 +1059,15 @@ static c_typedef_t* c_typedef_new( c_ast_t const *ast, unsigned gib_flags ) {
  * _must_ have its \ref predef_type::str "str" be NULL.
  */
 static void parse_predef_types( predef_type_t const types[static const 2] ) {
-  assert( types != NULL );
   for ( predef_type_t const *pt = types; pt->str != NULL; ++pt ) {
+    //
+    // Set Flex's current line number to the predefined type's source line
+    // number in this file.
+    //
+    yylineno = STATIC_CAST( int, pt->line );
+
     if ( unlikely( cdecl_parse_string( pt->str, strlen( pt->str ) ) != EX_OK ) )
-      INTERNAL_ERROR( "failed parsing predefined type on line %u\n", pt->line );
+      INTERNAL_ERROR( "failed parsing type predefined on line %u\n", pt->line );
   } // for
 }
 
@@ -1062,19 +1093,22 @@ static bool rb_visitor( void *node_data, void *v_data ) {
 
 ////////// extern functions ///////////////////////////////////////////////////
 
-rb_node_t* c_typedef_add( c_ast_t const *ast, unsigned gib_flags ) {
+rb_node_t* c_typedef_add( c_ast_t const *ast, decl_flags_t decl_flags ) {
   assert( ast != NULL );
   assert( !c_sname_empty( &ast->sname ) );
 
-  c_typedef_t *const new_tdef = c_typedef_new( ast, gib_flags );
-  rb_insert_rv_t const rbi = rb_tree_insert( &typedef_set, new_tdef );
-  if ( !rbi.inserted ) {
+  c_typedef_t find_tdef = { .ast = ast };
+  rb_insert_rv_t rbi_rv = rb_tree_insert( &typedef_set, &find_tdef );
+  if ( rbi_rv.inserted ) {
     //
-    // A typedef with the same name exists, so we don't need the new one.
+    // Now that we know the typedef has been inserted, replace the find_tdef
+    // key used to test for insertion with a dynamically allocated one.  Doing
+    // it this way means we do the look-up only once and the dynamic allocation
+    // only if inserted.
     //
-    FREE( new_tdef );
+    rbi_rv.node->data = c_typedef_new( ast, decl_flags );
   }
-  return rbi.node;
+  return rbi_rv.node;
 }
 
 c_typedef_t const* c_typedef_find_name( char const *name ) {
@@ -1085,29 +1119,44 @@ c_typedef_t const* c_typedef_find_name( char const *name ) {
     c_sname_cleanup( &sname );
     return tdef;
   }
-  return NULL;
+  return NULL;                          // LCOV_EXCL_LINE
 }
 
 c_typedef_t const* c_typedef_find_sname( c_sname_t const *sname ) {
   assert( sname != NULL );
-  rb_node_t const *const found_rb =
-    rb_tree_find( &typedef_set, &C_TYPEDEF_SNAME_LIT( *sname ) );
+  c_typedef_t const tdef = { .ast = &(c_ast_t const){ .sname = *sname } };
+  rb_node_t const *const found_rb = rb_tree_find( &typedef_set, &tdef );
   return found_rb != NULL ? found_rb->data : NULL;
 }
 
-void c_typedef_init( void ) {
+c_typedef_t* c_typedef_remove( rb_node_t *node ) {
+  return rb_tree_delete( &typedef_set, node );
+}
+
+void c_typedef_visit( c_typedef_visit_fn_t visit_fn, void *v_data ) {
+  assert( visit_fn != NULL );
+  tdef_rb_visit_data_t trvd = { visit_fn, v_data };
+  rb_tree_visit( &typedef_set, &rb_visitor, &trvd );
+}
+
+void c_typedefs_init( void ) {
   ASSERT_RUN_ONCE();
 
   rb_tree_init( &typedef_set, POINTER_CAST( rb_cmp_fn_t, &c_typedef_cmp ) );
-  check_atexit( &c_typedef_cleanup );
+  ATEXIT( &c_typedefs_cleanup );
 
-#ifdef ENABLE_CDECL_DEBUG
+#ifdef ENABLE_BISON_DEBUG
+  //
+  // Temporarily turn off Bison debug output for built-in typedefs.
+  //
+  int const orig_bison_debug = opt_bison_debug;
+  opt_bison_debug = false;
+#endif /* ENABLE_BISON_DEBUG */
   //
   // Temporarily turn off debug output for built-in typedefs.
   //
-  bool const orig_cdecl_debug = opt_cdecl_debug;
-  opt_cdecl_debug = false;
-#endif /* ENABLE_CDECL_DEBUG */
+  cdecl_debug_t const orig_cdecl_debug = opt_cdecl_debug;
+  opt_cdecl_debug = CDECL_DEBUG_NO;
 #ifdef ENABLE_FLEX_DEBUG
   //
   // Temporarily turn off Flex debug output for built-in typedefs.
@@ -1115,22 +1164,25 @@ void c_typedef_init( void ) {
   int const orig_flex_debug = opt_flex_debug;
   opt_flex_debug = false;
 #endif /* ENABLE_FLEX_DEBUG */
-#ifdef YYDEBUG
-  //
-  // Temporarily turn off Bison debug output for built-in typedefs.
-  //
-  int const orig_bison_debug = opt_bison_debug;
-  opt_bison_debug = false;
-#endif /* YYDEBUG */
 
-  c_lang_id_t const orig_lang = opt_lang;
+  //
+  // Temporarily set config_path to this file so if there's an error in a
+  // predefined type, print_loc() will print this file in the error message.
+  //
+  // Doing this requires that we also temporarily turn off command echoing.
+  //
+  cdecl_input_path = __FILE__;
+  bool const orig_echo_commands = opt_echo_commands;
+  opt_echo_commands = false;
+
+  c_lang_id_t const orig_lang_id = opt_lang_id;
 
   if ( opt_typedefs ) {
     //
     // Temporarily switch to the latest supported version of C so all keywords
     // will be available.
     //
-    opt_lang = LANG_C_NEW;
+    opt_lang_id = LANG_C_NEW;
 
     predef_lang_ids = LANG_MIN(C_KNR);
     parse_predef_types( PREDEFINED_KNR_C );
@@ -1149,10 +1201,10 @@ void c_typedef_init( void ) {
     parse_predef_types( PREDEFINED_STD_C_99 );
 
     // However, Embedded C extensions are available only in C99.
-    opt_lang = LANG_C_99;
+    opt_lang_id = LANG_C_99;
     predef_lang_ids = LANG_C_99;
     parse_predef_types( PREDEFINED_EMBEDDED_C );
-    opt_lang = LANG_C_NEW;
+    opt_lang_id = LANG_C_NEW;
 
     // Must be defined after C99.
     predef_lang_ids = LANG_MIN(C_89);
@@ -1169,7 +1221,7 @@ void c_typedef_init( void ) {
   // Temporarily switch to the latest supported version of C++ so all keywords
   // will be available.
   //
-  opt_lang = LANG_CPP_NEW;
+  opt_lang_id = LANG_CPP_NEW;
 
   if ( opt_typedefs ) {
     predef_lang_ids = LANG_MIN(CPP_OLD);
@@ -1190,31 +1242,19 @@ void c_typedef_init( void ) {
 
   predef_lang_ids = LANG_MIN(CPP_20);
   parse_predef_types( PREDEFINED_STD_CPP_20_REQUIRED );
-
   predef_lang_ids = LANG_NONE;
-  opt_lang = orig_lang;
 
-#ifdef ENABLE_CDECL_DEBUG
+#ifdef ENABLE_BISON_DEBUG
+  opt_bison_debug = orig_bison_debug;
+#endif /* ENABLE_BISON_DEBUG */
   opt_cdecl_debug = orig_cdecl_debug;
-#endif /* ENABLE_CDECL_DEBUG */
+  opt_echo_commands = orig_echo_commands;
 #ifdef ENABLE_FLEX_DEBUG
   opt_flex_debug = orig_flex_debug;
 #endif /* ENABLE_FLEX_DEBUG */
-#ifdef YYDEBUG
-  opt_bison_debug = orig_bison_debug;
-#endif /* YYDEBUG */
-}
+  opt_lang_id = orig_lang_id;
 
-c_typedef_t* c_typedef_remove( rb_node_t *node ) {
-  return rb_tree_delete( &typedef_set, node );
-}
-
-c_typedef_t const* c_typedef_visit( c_typedef_visit_fn_t visit_fn,
-                                    void *v_data ) {
-  assert( visit_fn != NULL );
-  tdef_rb_visit_data_t trvd = { visit_fn, v_data };
-  rb_node_t const *const rb = rb_tree_visit( &typedef_set, &rb_visitor, &trvd );
-  return rb != NULL ? rb->data : NULL;
+  cdecl_input_path = NULL;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
