@@ -2,7 +2,7 @@
 **      cdecl -- C gibberish translator
 **      src/cdeck_keyword.cpp
 **
-**      Copyright (C) 2021-2023  Paul J. Lucas
+**      Copyright (C) 2021-2024  Paul J. Lucas
 **
 **      This program is free software: you can redistribute it and/or modify
 **      it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 
 // standard
 #include <assert.h>
+#include <stddef.h>                     /* for NULL */
 #include <stdlib.h>
 #include <string.h>
 
@@ -107,7 +108,7 @@
 #define FIND_IN_ENGLISH_ONLY      false
 
 /**
- * Within \ref CDECL_KEYWORDS, specify that the previosuly given keyword is a
+ * Within \ref CDECL_KEYWORDS, specify that the previously given keyword is a
  * synonym for the given language-specific keywords.
  *
  * @param FIND_WHEN Either #ALWAYS_FIND or #FIND_IN_ENGLISH_ONLY.
@@ -119,7 +120,7 @@
  * @sa #TOKEN()
  */
 #define SYNONYMS(FIND_WHEN,...) \
-  (FIND_WHEN), /*y_token_id=*/0, C_LANG_LIT( __VA_ARGS__ )
+  (FIND_WHEN), .y_token_id = 0, C_LANG_LIT( __VA_ARGS__ )
 
 /**
  * Within \ref CDECL_KEYWORDS, a special-case of #SYNONYMS when there is only
@@ -138,7 +139,7 @@
   SYNONYMS( (FIND_WHEN), { LANG_ANY, (C_KEYWORD) } )
 
 /**
- * Within \ref CDECL_KEYWORDS, specify that the previosuly given keyword maps
+ * Within \ref CDECL_KEYWORDS, specify that the previously given keyword maps
  * to Bison tokan \a Y_ID.
  *
  * @param Y_ID The Bison token ID (`Y_xxx`).
@@ -148,16 +149,19 @@
  * @sa #SYNONYMS()
  */
 #define TOKEN(Y_ID) \
-  FIND_IN_ENGLISH_ONLY, (Y_ID), /*lang_syn=*/NULL
+  FIND_IN_ENGLISH_ONLY, (Y_ID), .lang_syn = NULL
 
 ///////////////////////////////////////////////////////////////////////////////
 
 /**
  * All **cdecl** keywords that are (mostly) _not_ C/C++ keywords.
  *
- * @note Exceptions are `alignas`, `bool`, `complex`, `const`, and `volatile`
- * that are included here as **cdecl** keywords so each maps to its language-
- * specific literal, e.g., `_Alignas` in C vs. `alignas` in C++.
+ * @note Exceptions are:
+ *  + `alignas`, `bool`, `complex`, `const`, and `volatile` so they can map to
+ *    their language-specific literals, e.g., `_Alignas` in C vs. `alignas` in
+ *    C++.
+ *  + `double` so it can specify the #AC_NEXT_KEYWORDS() of `precision`.
+ *
  * @note This is not declared `const` because it's sorted once.
  *
  * ## Initialization Macros
@@ -203,7 +207,7 @@
  *          // The "WINAPI" literal is always a synonym for the "__stdcall"
  *          // literal.
  *          { L_MSC_WINAPI,
- *            LANG_MSC_EXTENSIONS,
+ *            LANG_MSC_CALL_CONVS,
  *            SYNONYM( ALWAYS_FIND, L_MSC___stdcall ),
  *            ...
  *          }
@@ -215,7 +219,7 @@
  *          // In languages that do not support the `noreturn` keyword/macro,
  *          // it's a synonym for `_Noreturn`; otherwise it just maps to itself.
  *          { L_noreturn,
- *            LANG_NONRETURNING_FUNC,
+ *            LANG_NONRETURNING_FUNCS,
  *            SYNONYMS( ALWAYS_FIND,
  *              { ~LANG_noreturn, L__Noreturn },
  *              { LANG_ANY,       L_noreturn  } ),
@@ -294,7 +298,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     TOKEN( Y_array ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
-      AC_NEXT_KEYWORDS( L_const, L_static, L_volatile )
+      AC_NO_NEXT_KEYWORDS
     )
   },
 
@@ -325,6 +329,16 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  {
+    L_binding,
+    LANG_STRUCTURED_BINDINGS,
+    TOKEN( Y_binding ),
+    AC_SETTINGS(
+      AC_POLICY_IN_NEXT_ONLY,
+      AC_NO_NEXT_KEYWORDS
+    )
+  },
+
   { L_bit,
     LANG__BitInt,
     TOKEN( Y_bit ),
@@ -362,7 +376,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_bool,
-    LANG_BOOL_TYPE,
+    LANG_BOOLEAN,
     SYNONYMS( ALWAYS_FIND,
       { LANG__Bool, L__Bool },
       { LANG_ANY,   L_bool  } ),
@@ -373,7 +387,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_boolean,
-    LANG_BOOL_TYPE,
+    LANG_BOOLEAN,
     SYNONYMS( ALWAYS_FIND,
       { LANG__Bool, L__Bool },
       { LANG_ANY,   L_bool  } ),
@@ -384,7 +398,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_Boolean,
-    LANG_BOOL_TYPE,
+    LANG_BOOLEAN,
     SYNONYMS( ALWAYS_FIND,
       { LANG__Bool, L__Bool },
       { LANG_ANY,   L_bool  } ),
@@ -395,7 +409,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_by,
-    LANG_LAMBDA,
+    LANG_LAMBDAS,
     TOKEN( Y_by ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -413,7 +427,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_capture,
-    LANG_LAMBDA,
+    LANG_LAMBDAS,
     TOKEN( Y_capturing ),
     AC_SETTINGS(
       AC_POLICY_DEFER,                  // to "capturing"
@@ -422,7 +436,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_captures,
-    LANG_LAMBDA,
+    LANG_LAMBDAS,
     TOKEN( Y_capturing ),
     AC_SETTINGS(
       AC_POLICY_DEFER,                  // to "capturing"
@@ -431,7 +445,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_capturing,
-    LANG_LAMBDA,
+    LANG_LAMBDAS,
     TOKEN( Y_capturing ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -582,7 +596,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_constructor,
-    LANG_CPP_ANY,
+    LANG_CONSTRUCTORS,
     TOKEN( Y_constructor ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -609,7 +623,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_copy,
-    LANG_LAMBDA,
+    LANG_LAMBDAS,
     TOKEN( Y_copy ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -618,7 +632,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_ctor,
-    LANG_CPP_ANY,
+    LANG_CONSTRUCTORS,
     TOKEN( Y_constructor ),
     AC_SETTINGS(
       AC_POLICY_TOO_SHORT,
@@ -636,7 +650,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_defaulted,
-    LANG_default_delete_FUNC,
+    LANG_default_delete_FUNCS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_default ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -663,7 +677,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_deleted,
-    LANG_default_delete_FUNC,
+    LANG_default_delete_FUNCS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_delete ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -681,7 +695,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_destructor,
-    LANG_CPP_ANY,
+    LANG_CONSTRUCTORS,
     TOKEN( Y_destructor ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -698,6 +712,15 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  { L_double,
+    LANG_ANY,
+    SYNONYM( FIND_IN_ENGLISH_ONLY, L_double ),
+    AC_SETTINGS(
+      AC_POLICY_DEFER,                  // to C keyword
+      AC_NEXT_KEYWORDS( L_precision )
+    )
+  },
+
   { H_double_precision,
     LANG_ANY,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_double ),
@@ -708,7 +731,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_dtor,
-    LANG_CPP_ANY,
+    LANG_CONSTRUCTORS,
     TOKEN( Y_destructor ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -717,7 +740,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_dynamic,
-    LANG_CPP_ANY,
+    LANG_NEW_STYLE_CASTS,
     TOKEN( Y_dynamic ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -779,6 +802,15 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  { L_expand,
+    LANG_ANY,
+    TOKEN( Y_expand ),
+    AC_SETTINGS(
+      AC_POLICY_DEFAULT,
+      AC_NO_NEXT_KEYWORDS
+    )
+  },
+
   { L_explain,
     LANG_ANY,
     TOKEN( Y_explain ),
@@ -862,7 +894,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
 
   { L_help,
     LANG_ANY,
-    TOKEN( Y_help ),
+    TOKEN( '?' ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
       AC_NO_NEXT_KEYWORDS               // see command_ac_keywords()
@@ -918,16 +950,16 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
 
   {
     L_lambda,
-    LANG_LAMBDA,
+    LANG_LAMBDAS,
     TOKEN( Y_lambda ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
-      AC_NEXT_KEYWORDS( L_capturing, L_copy, L_reference )
+      AC_NEXT_KEYWORDS( L_capturing, L_copy, L_reference, L_returning )
     )
   },
 
   { L_len,
-    LANG_VLA,
+    LANG_VLAS,
     TOKEN( Y_length ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -936,7 +968,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_length,
-    LANG_VLA,
+    LANG_VLAS,
     TOKEN( Y_length ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -945,7 +977,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_linkage,
-    LANG_CPP_ANY,
+    LANG_LINKAGE_DECLS,
     TOKEN( Y_linkage ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -954,7 +986,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_literal,
-    LANG_USER_DEFINED_LITERAL,
+    LANG_USER_DEF_LITS,
     TOKEN( Y_literal ),
     AC_SETTINGS(
       AC_POLICY_IN_NEXT_ONLY,
@@ -971,6 +1003,15 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  { L_macros,
+    LANG_ANY,
+    TOKEN( Y_macros ),
+    AC_SETTINGS(
+      AC_POLICY_DEFAULT,
+      AC_NO_NEXT_KEYWORDS               // see command_ac_keywords()
+    )
+  },
+
   { L_maybe,
     LANG_maybe_unused,
     TOKEN( Y_maybe ),
@@ -1063,6 +1104,16 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  {
+    H_non_empty,
+    LANG_QUALIFIED_ARRAYS,
+    TOKEN( Y_non_empty ),
+    AC_SETTINGS(
+      AC_POLICY_DEFAULT,
+      AC_NEXT_KEYWORDS( L_array, L_const, L_restrict, L_variable, L_volatile )
+    )
+  },
+
   { H_non_mbr,
     LANG_CPP_ANY,
     TOKEN( Y_non_member ),
@@ -1082,7 +1133,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { H_non_returning,
-    LANG_NONRETURNING_FUNC,
+    LANG_NONRETURNING_FUNCS,
     SYNONYMS( FIND_IN_ENGLISH_ONLY,
       { ~LANG_noreturn, L__Noreturn },
       { LANG_ANY,       L_noreturn  } ),
@@ -1094,7 +1145,9 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
 
   { H_non_throwing,
     LANG_CPP_ANY,
-    SYNONYM( FIND_IN_ENGLISH_ONLY, L_throw ),
+    SYNONYMS( FIND_IN_ENGLISH_ONLY,
+      { LANG_noexcept, L_noexcept },
+      { LANG_ANY,      L_throw    } ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
       AC_NO_NEXT_KEYWORDS
@@ -1120,7 +1173,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_noreturn,
-    LANG_NONRETURNING_FUNC,
+    LANG_NONRETURNING_FUNCS,
     SYNONYMS( ALWAYS_FIND,
       { ~LANG_noreturn, L__Noreturn },
       { LANG_ANY,       L_noreturn  } ),
@@ -1140,7 +1193,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_oper,
-    LANG_CPP_ANY,
+    LANG_operator,
     TOKEN( Y_operator ),
     AC_SETTINGS(
       AC_POLICY_DEFER,                  // to "operator"
@@ -1166,6 +1219,24 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  { L_pack,
+    LANG_PARAMETER_PACKS,
+    TOKEN( Y_pack ),
+    AC_SETTINGS(
+      AC_POLICY_IN_NEXT_ONLY,
+      AC_NO_NEXT_KEYWORDS
+    )
+  },
+
+  { L_parameter,
+    LANG_PARAMETER_PACKS,
+    TOKEN( Y_parameter ),
+    AC_SETTINGS(
+      AC_POLICY_DEFAULT,
+      AC_NEXT_KEYWORDS( L_pack )
+    )
+  },
+
   { L_point,
     LANG_ANY,
     TOKEN( Y_point ),
@@ -1230,7 +1301,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_pure,
-    LANG_CPP_ANY,
+    LANG_virtual,
     TOKEN( Y_pure ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1248,7 +1319,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_ref,
-    LANG_CPP_ANY,
+    LANG_REFERENCES,
     TOKEN( Y_reference ),
     AC_SETTINGS(
       AC_POLICY_DEFER,                  // to "reference"
@@ -1257,7 +1328,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_reference,
-    LANG_CPP_ANY,
+    LANG_REFERENCES,
     TOKEN( Y_reference ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1266,7 +1337,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_reinterpret,
-    LANG_CPP_ANY,
+    LANG_NEW_STYLE_CASTS,
     TOKEN( Y_reinterpret ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1304,7 +1375,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_rvalue,
-    LANG_RVALUE_REFERENCE,
+    LANG_RVALUE_REFERENCES,
     TOKEN( Y_rvalue ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1313,7 +1384,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_scope,
-    LANG_CPP_ANY,
+    LANG_SCOPED_NAMES,
     TOKEN( Y_scope ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1348,6 +1419,15 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
     )
   },
 
+  { L_structured,
+    LANG_STRUCTURED_BINDINGS,
+    TOKEN( Y_structured ),
+    AC_SETTINGS(
+      AC_POLICY_DEFAULT,
+      AC_NEXT_KEYWORDS( L_binding )
+    )
+  },
+
   { L_thread,
     LANG_THREAD_LOCAL_STORAGE,
     TOKEN( Y_thread ),
@@ -1376,7 +1456,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
       //     cdecl> explain thread_local int x
       //     declare x as thread local integer
       //
-      // would be legal (becasuse `thread_local` would map to GNU C's
+      // would be legal (because `thread_local` would map to GNU C's
       // `__thread` that's legal in all languages) when it shouldn't be
       // (because neither the C macro `thread_local` nor the C keyword
       // `_Thread_local` were supported until C11).
@@ -1465,7 +1545,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_var,
-    LANG_VLA,
+    LANG_VLAS,
     TOKEN( Y_variable ),
     AC_SETTINGS(
       AC_POLICY_DEFER,                  // to "variable"
@@ -1483,7 +1563,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_variable,
-    LANG_VLA,
+    LANG_VLAS,
     TOKEN( Y_variable ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1585,7 +1665,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
 
   // Microsoft extensions
   { L_MSC_cdecl,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_MSC___cdecl ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1594,7 +1674,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_MSC_clrcall,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_MSC___clrcall ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1603,7 +1683,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_MSC_fastcall,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_MSC___fastcall ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1612,7 +1692,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_MSC_stdcall,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_MSC___stdcall ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1621,7 +1701,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_MSC_thiscall,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_MSC___thiscall ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1630,7 +1710,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_MSC_vectorcall,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( FIND_IN_ENGLISH_ONLY, L_MSC___vectorcall ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1639,7 +1719,7 @@ static cdecl_keyword_t CDECL_KEYWORDS[] = {
   },
 
   { L_MSC_WINAPI,
-    LANG_MSC_EXTENSIONS,
+    LANG_MSC_CALL_CONVS,
     SYNONYM( ALWAYS_FIND, L_MSC___stdcall ),
     AC_SETTINGS(
       AC_POLICY_DEFAULT,
@@ -1675,29 +1755,12 @@ static int cdecl_keyword_cmp( cdecl_keyword_t const *i_cdk,
 
 ////////// extern functions ///////////////////////////////////////////////////
 
-cdecl_keyword_t const* cdecl_keyword_find( char const *s ) {
-  assert( s != NULL );
-
-  // the list is small, so linear search is good enough
-  for ( cdecl_keyword_t const *cdk = CDECL_KEYWORDS; cdk->literal != NULL;
-        ++cdk ) {
-    int const cmp = strcmp( s, cdk->literal );
-    if ( cmp > 0 )
-      continue;
-    if ( cmp < 0 )                      // the array is sorted
-      break;
-    return cdk;
-  } // for
-
-  return NULL;
-}
-
-void cdecl_keyword_init( void ) {
-  ASSERT_RUN_ONCE();
-  qsort(                                // don't rely on manual sorting above
-    CDECL_KEYWORDS, ARRAY_SIZE( CDECL_KEYWORDS ) - 1/*NULL*/,
-    sizeof( cdecl_keyword_t ),
-    POINTER_CAST( qsort_cmp_fn_t, &cdecl_keyword_cmp )
+cdecl_keyword_t const* cdecl_keyword_find( char const *literal ) {
+  assert( literal != NULL );
+  return bsearch(
+    &(cdecl_keyword_t){ .literal = literal }, CDECL_KEYWORDS,
+    ARRAY_SIZE( CDECL_KEYWORDS ) - 1/*NULL*/, sizeof( CDECL_KEYWORDS[0] ),
+    POINTER_CAST( bsearch_cmp_fn_t, &cdecl_keyword_cmp )
   );
 }
 
@@ -1705,6 +1768,15 @@ cdecl_keyword_t const* cdecl_keyword_next( cdecl_keyword_t const *cdk ) {
   return cdk == NULL ? CDECL_KEYWORDS : (++cdk)->literal == NULL ? NULL : cdk;
 }
 
+void cdecl_keywords_init( void ) {
+  ASSERT_RUN_ONCE();
+  qsort(                                // don't rely on manual sorting above
+    CDECL_KEYWORDS,
+    ARRAY_SIZE( CDECL_KEYWORDS ) - 1/*NULL*/, sizeof( CDECL_KEYWORDS[0] ),
+    POINTER_CAST( qsort_cmp_fn_t, &cdecl_keyword_cmp )
+  );
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 /** @} */
