@@ -2,7 +2,7 @@
 **      cdecl -- C gibberish translator
 **      src/literals.c
 **
-**      Copyright (C) 2017-2023  Paul J. Lucas
+**      Copyright (C) 2017-2024  Paul J. Lucas
 **
 **      This program is free software: you can redistribute it and/or modify
 **      it under the terms of the GNU General Public License as published by
@@ -58,6 +58,7 @@ char const L_english[]                  = "english";
 char const L_eval[]                     = "eval";
 char const L_evaluation[]               = "evaluation";
 char const L_exit[]                     = "exit";
+char const L_expand[]                   = "expand";
 char const L_explain[]                  = "explain";
 char const L_expr[]                     = "expr";
 char const L_expression[]               = "expression";
@@ -70,9 +71,11 @@ char const L_init[]                     = "init";
 char const L_initialization[]           = "initialization";
 char const L_into[]                     = "into";
 char const L_linkage[]                  = "linkage";
+char const L_macros[]                   = "macros";
 char const L_mbr[]                      = "mbr";
 char const L_member[]                   = "member";
 char const L_no[]                       = "no";
+char const H_non_empty[]                = "non-empty";
 char const H_non_mbr[]                  = "non-mbr";
 char const H_non_member[]               = "non-member";
 char const L_of[]                       = "of";
@@ -97,6 +100,65 @@ char const L_vector[]                   = "vector";
 char const L_wide[]                     = "wide";
 char const L_width[]                    = "width";
 
+// cdecl options
+char const L_OPT_alt_tokens[]           = "alt-tokens";
+#ifdef ENABLE_BISON_DEBUG
+char const L_OPT_bison_debug[]          = "bison-debug";
+#endif /* ENABLE_BISON_DEBUG */
+char const L_OPT_color[]                = "color";
+char const L_OPT_commands[]             = "commands";
+char const L_OPT_config[]               = "config";
+char const L_OPT_debug[]                = "debug";
+char const L_OPT_digraphs[]             = "digraphs";
+char const L_OPT_east_const[]           = "east-const";
+char const L_OPT_echo_commands[]        = "echo-commands";
+char const L_OPT_english_types[]        = "english-types";
+char const L_OPT_explicit_ecsu[]        = "explicit-ecsu";
+char const L_OPT_explicit_int[]         = "explicit-int";
+char const L_OPT_file[]                 = "file";
+#ifdef ENABLE_FLEX_DEBUG
+char const L_OPT_flex_debug[]           = "flex-debug";
+#endif /* ENABLE_FLEX_DEBUG */
+char const L_OPT_help[]                 = "help";
+char const L_OPT_infer_command[]        = "infer-command";
+char const L_OPT_language[]             = "language";
+char const L_OPT_lineno[]               = "lineno";
+char const L_OPT_options[]              = "options";
+char const L_OPT_output[]               = "output";
+char const L_OPT_permissive_types[]     = "permissive-types";
+char const L_OPT_prompt[]               = "prompt";
+char const L_OPT_semicolon[]            = "semicolon";
+char const L_OPT_trailing_return[]      = "trailing-return";
+char const L_OPT_trigraphs[]            = "trigraphs";
+char const L_OPT_using[]                = "using";
+char const L_OPT_version[]              = "version";
+char const L_OPT_west_decl[]            = "west-decl";
+
+// C Preprocessor
+char const L_PRE_define[]               = "define";
+char const L_PRE_elif[]                 = "elif";
+char const L_PRE_else[]                 = "else";
+char const L_PRE_error[]                = "error";
+char const L_PRE_if[]                   = "if";
+char const L_PRE_ifdef[]                = "ifdef";
+char const L_PRE_ifndef[]               = "ifndef";
+//         L_PRE_include[]              // handled within the lexer
+char const L_PRE_line[]                 = "line";
+char const L_PRE_undef[]                = "undef";
+
+char const L_PRE_P_define[]             = "#define";
+char const L_PRE_P_include[]            = "#include";
+char const L_PRE_P_undef[]              = "#undef";
+
+// C99 Preprocessor
+char const L_PRE_pragma[]               = "pragma";
+
+// C23 Preprocessor
+char const L_PRE_elifdef[]              = "elifdef";
+char const L_PRE_elifndef[]             = "elifndef";
+char const L_PRE_embed[]                = "embed";
+char const L_PRE_warning[]              = "warning";
+
 // K&R
 char const L_auto[]                     = "auto";
 char const L_automatic[]                = "automatic";
@@ -138,7 +200,7 @@ char const L_while[]                    = "while";
 char const L_asm[]                      = "asm";
 char const L_const[]                    = "const";
 char const L_constant[]                 = "constant";
-char const L_ellipsis[]                 = "...";
+char const L_ELLIPSIS[]                 = "...";
 char const L_enum[]                     = "enum";
 char const L_enumeration[]              = "enumeration";
 char const L_signed[]                   = "signed";
@@ -162,6 +224,9 @@ char const L_var[]                      = "var";
 char const L_variable[]                 = "variable";
 char const L_wchar_t[]                  = "wchar_t";
 
+// C99 Preprocessor
+char const L_PRE___VA_ARGS__[]          = "__VA_ARGS__";
+
 // C11
 char const L__Alignas[]                 = "_Alignas";
 char const L__Alignof[]                 = "_Alignof";
@@ -263,6 +328,9 @@ char const L_thread_local[]             = "thread_local";
 char const L_deprecated[]               = "deprecated";
 char const L___deprecated__[]           = "__deprecated__";
 
+// C23 & C++20 Preprocessor
+char const L_PRE___VA_OPT__[]           = "__VA_OPT__";
+
 // C++17
 char const L_maybe_unused[]             = "maybe_unused";
 char const L_maybe[]                    = "maybe", L_unused[] = "unused";
@@ -273,6 +341,8 @@ char const L_discard[]                  = "discard";
 char const H_no_discard[]               = "no-discard";
 char const L___nodiscard__[]            = "__nodiscard__";
 char const H_non_discardable[]          = "non-discardable";
+char const L_structured[]               = "structured";
+char const L_binding[]                  = "binding";
 
 // C++20
 char const L_char8_t[]                  = "char8_t";
@@ -292,6 +362,8 @@ char const L_no_unique_address[]        = "no_unique_address";
 char const L_unique[]                   = "unique", L_address[] = "address";
 char const H_no_unique_address[]        = "no-unique-address";
 char const H_non_unique_address[]       = "non-unique-address";
+char const L_pack[]                     = "pack";
+char const L_parameter[]                = "parameter";
 char const L_requires[]                 = "requires";
 
 // Alternative tokens
@@ -334,6 +406,7 @@ char const L_GNU___restrict__[]         = "__restrict__";
 char const L_GNU___signed[]             = "__signed";
 char const L_GNU___signed__[]           = "__signed__";
 char const L_GNU___thread[]             = "__thread";
+char const L_GNU___typeof__[]           = "__typeof__";
 char const L_GNU___volatile[]           = "__volatile";
 char const L_GNU___volatile__[]         = "__volatile__";
 
