@@ -2,7 +2,7 @@
 **      cdecl -- C gibberish translator
 **      src/slist_test.c
 **
-**      Copyright (C) 2021-2023  Paul J. Lucas
+**      Copyright (C) 2021-2024  Paul J. Lucas
 **
 **      This program is free software: you can redistribute it and/or modify
 **      it under the terms of the GNU General Public License as published by
@@ -26,63 +26,59 @@
 
 // standard
 #include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
-#include <stdnoreturn.h>
 #include <string.h>
 #include <sysexits.h>
 
 ///////////////////////////////////////////////////////////////////////////////
 
-// extern variables
-char const          *me;                ///< Program name.
-
 // local variables
 static char const  *str_equal_to;
-static unsigned     test_failures;
 
 ////////// local functions ////////////////////////////////////////////////////
 
-static bool slist_node_str_equal( void *data ) {
-  char const *const s = data;
+static bool slist_node_str_equal( slist_node_t *node, void *data ) {
+  (void)data;
+  char const *const s = node->data;
   return strcmp( s, str_equal_to ) == 0;
 }
 
-static bool test_slist_cmp( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_cmp( void ) {
   slist_t list, list2;
   slist_init( &list );
   slist_init( &list2 );
 
   // test 2 empty lists
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) == 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) == 0 );
 
   // test empty and non-empty lists
   slist_push_back( &list2, (void*)"A" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) != 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) != 0 );
   slist_cleanup( &list2, /*free_fn=*/NULL );
 
   // test non-empty and empty lists
   slist_push_back( &list, (void*)"A" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) != 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) != 0 );
   slist_cleanup( &list, /*free_fn=*/NULL );
 
   // test matching 1,2,3-element lists
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list2, (void*)"A" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) == 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) == 0 );
   slist_push_back( &list, (void*)"B" );
   slist_push_back( &list2, (void*)"B" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) == 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) == 0 );
   slist_push_back( &list, (void*)"C" );
   slist_push_back( &list2, (void*)"C" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) == 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) == 0 );
   slist_cleanup( &list, /*free_fn=*/NULL );
   slist_cleanup( &list2, /*free_fn=*/NULL );
 
   // test 1-element non-matching lists
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list2, (void*)"B" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) < 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) < 0 );
   slist_cleanup( &list, /*free_fn=*/NULL );
   slist_cleanup( &list2, /*free_fn=*/NULL );
 
@@ -90,15 +86,12 @@ static bool test_slist_cmp( void ) {
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list2, (void*)"A" );
   slist_push_back( &list2, (void*)"B" );
-  TEST( slist_cmp( &list, &list2, (slist_cmp_fn_t)&strcmp ) < 0 );
+  TEST( slist_cmp( &list, &list2, POINTER_CAST( slist_cmp_fn_t, &strcmp ) ) < 0 );
   slist_cleanup( &list, /*free_fn=*/NULL );
   slist_cleanup( &list2, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
-static bool test_slist_dup( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_dup( void ) {
   slist_t list, list2;
   slist_init( &list );
   slist_init( &list2 );
@@ -153,7 +146,7 @@ static bool test_slist_dup( void ) {
 
   // check data_dup_fn
   slist_push_back( &list, (void*)&X );
-  list2 = slist_dup( &list, -1, (slist_dup_fn_t)&strdup );
+  list2 = slist_dup( &list, -1, POINTER_CAST( slist_dup_fn_t, &strdup ) );
   TEST( !slist_empty( &list2 ) );
   TEST( slist_len( &list2 ) == 1 );
   if ( TEST( (p = slist_front( &list2 )) != NULL ) ) {
@@ -163,12 +156,9 @@ static bool test_slist_dup( void ) {
   }
   slist_cleanup( &list, /*free_fn=*/NULL );
   slist_cleanup( &list2, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
-static bool test_slist_free_if( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_free_if( void ) {
   slist_t list;
   slist_init( &list );
   char *p;
@@ -176,21 +166,21 @@ static bool test_slist_free_if( void ) {
   // test match list[0] with list->len == 1
   slist_push_back( &list, (void*)"A" );
   str_equal_to = "A";
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_empty( &list ) );
   TEST( slist_len( &list ) == 0 );
 
   // test match list[0] and list[1] with list->len == 2
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list, (void*)"A" );
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_empty( &list ) );
   TEST( slist_len( &list ) == 0 );
 
   // test match list[0] with list->len == 2
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list, (void*)"B" );
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_len( &list ) == 1 );
   if ( TEST( (p = slist_front( &list )) != NULL ) )
     TEST( *p == 'B' );
@@ -202,7 +192,7 @@ static bool test_slist_free_if( void ) {
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list, (void*)"B" );
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_len( &list ) == 1 );
   if ( TEST( (p = slist_front( &list )) != NULL ) )
     TEST( *p == 'B' );
@@ -215,7 +205,7 @@ static bool test_slist_free_if( void ) {
   slist_push_back( &list, (void*)"B" );
   slist_push_back( &list, (void*)"C" );
   str_equal_to = "B";
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_len( &list ) == 2 );
   if ( TEST( (p = slist_front( &list )) != NULL ) )
     TEST( *p == 'A' );
@@ -226,7 +216,7 @@ static bool test_slist_free_if( void ) {
   // test match list[1] with list->len == 2
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list, (void*)"B" );
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_len( &list ) == 1 );
   if ( TEST( (p = slist_front( &list )) != NULL ) )
     TEST( *p == 'A' );
@@ -238,19 +228,16 @@ static bool test_slist_free_if( void ) {
   slist_push_back( &list, (void*)"A" );
   slist_push_back( &list, (void*)"B" );
   slist_push_back( &list, (void*)"B" );
-  slist_free_if( &list, &slist_node_str_equal );
+  slist_free_if( &list, &slist_node_str_equal, /*data=*/NULL );
   TEST( slist_len( &list ) == 1 );
   if ( TEST( (p = slist_front( &list )) != NULL ) )
     TEST( *p == 'A' );
   if ( TEST( (p = slist_back( &list )) != NULL ) )
     TEST( *p == 'A' );
   slist_cleanup( &list, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
-static bool test_slist_at( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_at( void ) {
   slist_t list;
   slist_init( &list );
   char *p;
@@ -266,12 +253,9 @@ static bool test_slist_at( void ) {
     TEST( *p == 'C' );
   TEST( slist_at( &list, 3 ) == NULL );
   slist_cleanup( &list, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
-static bool test_slist_atr( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_atr( void ) {
   slist_t list;
   slist_init( &list );
   char *p;
@@ -287,12 +271,9 @@ static bool test_slist_atr( void ) {
     TEST( *p == 'A' );
   TEST( slist_at( &list, 4 ) == NULL );
   slist_cleanup( &list, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
-static bool test_slist_pop_front( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_pop_front( void ) {
   slist_t list;
   slist_init( &list );
   char *p;
@@ -313,12 +294,11 @@ static bool test_slist_pop_front( void ) {
   }
   TEST( slist_pop_front( &list ) == NULL );
   slist_cleanup( &list, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
 static bool test_slist_push_front( void ) {
-  TEST_FN_BEGIN();
+  TEST_FUNC_BEGIN();
+
   slist_t list;
   slist_init( &list );
   char *p;
@@ -336,11 +316,10 @@ static bool test_slist_push_front( void ) {
     TEST( *p == 'B' );
   slist_cleanup( &list, /*free_fn=*/NULL );
 
-  TEST_FN_END();
+  TEST_FUNC_END();
 }
 
-static bool test_slist_push_list_front( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_push_list_front( void ) {
   slist_t list;
   slist_init( &list );
   char *p;
@@ -470,12 +449,9 @@ static bool test_slist_push_list_front( void ) {
   if ( TEST( (p = slist_back( &list )) != NULL ) )
     TEST( *p == 'E' );
   slist_cleanup( &list, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
-static bool test_slist_push_list_back( void ) {
-  TEST_FN_BEGIN();
+static void test_slist_push_list_back( void ) {
   slist_t list;
   slist_init( &list );
   char *p;
@@ -605,12 +581,10 @@ static bool test_slist_push_list_back( void ) {
   if ( TEST( (p = slist_back( &list )) != NULL ) )
     TEST( *p == 'E' );
   slist_cleanup( &list, /*free_fn=*/NULL );
-
-  TEST_FN_END();
 }
 
 static bool test_slist_push_back( void ) {
-  TEST_FN_BEGIN();
+  TEST_FUNC_BEGIN();
   slist_t list;
   slist_init( &list );
   char *p;
@@ -631,37 +605,24 @@ static bool test_slist_push_back( void ) {
     TEST( *p == 'B' );
   slist_cleanup( &list, /*free_fn=*/NULL );
 
-  TEST_FN_END();
+  TEST_FUNC_END();
 }
 
-// LCOV_EXCL_START
-static noreturn void usage( void ) {
-  EPRINTF( "usage: %s\n", me );
-  exit( EX_USAGE );
-}
-// LCOV_EXCL_STOP
-
 ////////// main ///////////////////////////////////////////////////////////////
 
-int main( int argc, char const *argv[const] ) {
-  me = base_name( argv[0] );
-  if ( --argc != 0 )
-    usage();                            // LCOV_EXCL_LINE
+int main( int argc, char const *const argv[] ) {
+  test_prog_init( argc, argv );
 
   if ( test_slist_push_front() && test_slist_push_back() ) {
     test_slist_free_if();
-    if ( test_slist_at() ) {
-      test_slist_atr();
-      test_slist_cmp();
-      test_slist_dup();
-      test_slist_push_list_front();
-      test_slist_push_list_back();
-    }
+    test_slist_at();
+    test_slist_atr();
+    test_slist_cmp();
+    test_slist_dup();
+    test_slist_push_list_front();
+    test_slist_push_list_back();
     test_slist_pop_front();
   }
-
-  printf( "%u failures\n", test_failures );
-  exit( test_failures > 0 ? EX_SOFTWARE : EX_OK );
 }
 
 ///////////////////////////////////////////////////////////////////////////////
